import platform
from os import chdir

import pytest

from conan.tools.files.files import load_toolchain_args
from conan.tools.gnu import AutotoolsToolchain
from conans.errors import ConanException
from conans.model.conf import Conf
from conans.test.utils.mocks import ConanFileMock, MockSettings, MockOptions
from conans.test.utils.test_files import temp_folder


def test_modify_environment():
    """We can alter the environment generated by the toolchain passing the env to the generate"""
    f = temp_folder()
    chdir(f)
    conanfile = ConanFileMock()
    conanfile.settings = MockSettings({"os": "Linux", "arch": "x86_64"})
    conanfile.settings_build = MockSettings({"os": "Solaris", "arch": "x86"})
    conanfile.folders.set_base_install(f)
    be = AutotoolsToolchain(conanfile)
    env = be.environment()
    env.define("foo", "var")
    # We can pass the env to the generate once we adjusted or injected anything
    be.generate(env)

    with open("conanautotoolstoolchain.sh") as f:
        content = f.read()
        assert "foo" in content


def test_target_triple():
    f = temp_folder()
    chdir(f)
    conanfile = ConanFileMock()
    conanfile.settings = MockSettings({"os": "Linux", "arch": "x86_64"})
    conanfile.settings_build = MockSettings({"os": "Solaris", "arch": "x86"})
    conanfile.conf = Conf()
    conanfile.conf["tools.gnu:make_program"] = "my_make"
    conanfile.conf["tools.gnu.make:jobs"] = "23"

    be = AutotoolsToolchain(conanfile)
    be.make_args = ["foo", "var"]
    be.generate_args()
    obj = load_toolchain_args()
    assert "--host=x86_64-linux-gnu" in obj["configure_args"]
    assert "--build=i686-solaris" in obj["configure_args"]
    assert obj["make_args"].replace("'", "") == "foo var"


def test_invalid_target_triple():
    conanfile = ConanFileMock()
    conanfile.settings = MockSettings({"os": "Linux", "arch": "UNKNOWN_ARCH"})
    conanfile.settings_build = MockSettings({"os": "Solaris", "arch": "x86"})
    with pytest.raises(ConanException) as excinfo:
        AutotoolsToolchain(conanfile)
    assert "Unknown 'UNKNOWN_ARCH' machine, Conan doesn't know how " \
           "to translate it to the GNU triplet," in str(excinfo)


def test_cppstd():
    # Using "cppstd" is discarded
    conanfile = ConanFileMock()
    conanfile.settings = MockSettings(
        {"build_type": "Release",
         "arch": "x86",
         "compiler": "gcc",
         "compiler.libcxx": "libstdc++11",
         "compiler.version": "7.1",
         "cppstd": "17"})
    be = AutotoolsToolchain(conanfile)
    env = be.vars()
    assert "-std=c++17" not in env["CXXFLAGS"]

    # Using "compiler.cppstd" works
    conanfile.settings = MockSettings(
        {"build_type": "Release",
         "arch": "x86",
         "compiler": "gcc",
         "compiler.libcxx": "libstdc++11",
         "compiler.version": "7.1",
         "compiler.cppstd": "17"})
    be = AutotoolsToolchain(conanfile)
    env = be.vars()
    assert "-std=c++17" in env["CXXFLAGS"]

    # With visual
    conanfile.settings = MockSettings(
        {"build_type": "Release",
         "arch": "x86",
         "compiler": "Visual Studio",
         "compiler.version": "14",
         "compiler.cppstd": "17"})
    be = AutotoolsToolchain(conanfile)
    env = be.vars()
    assert "/std:c++latest" in env["CXXFLAGS"]

    # With MSVC
    conanfile.settings = MockSettings(
        {"build_type": "Release",
         "arch": "x86",
         "compiler": "msvc",
         "compiler.version": "193",
         "compiler.cppstd": "17"})
    be = AutotoolsToolchain(conanfile)
    env = be.vars()
    assert "/std:c++17" in env["CXXFLAGS"]


def test_fpic():
    conanfile = ConanFileMock()
    conanfile.settings = MockSettings({"os": "Linux"})
    conanfile.options = MockOptions({"fPIC": True})
    be = AutotoolsToolchain(conanfile)
    be.vars()
    assert be.fpic is True
    assert "-fPIC" in be.cxxflags

    conanfile.options = MockOptions({"fPIC": False})
    be = AutotoolsToolchain(conanfile)
    be.vars()
    assert be.fpic is False
    assert "-fPIC" not in be.cxxflags

    conanfile.options = MockOptions({"shared": False})
    be = AutotoolsToolchain(conanfile)
    be.vars()
    assert be.fpic is None
    assert "-fPIC" not in be.cxxflags


def test_ndebug():
    conanfile = ConanFileMock()
    for bt in ['Release', 'RelWithDebInfo', 'MinSizeRel']:
        conanfile.settings = MockSettings({"build_type": bt})
        be = AutotoolsToolchain(conanfile)
        assert be.ndebug == "NDEBUG"
        env = be.vars()
        assert "-DNDEBUG" in env["CPPFLAGS"]
    for bt in ['Debug', 'DebWithDebInfo']:
        conanfile.settings = MockSettings({"build_type": bt})
        be = AutotoolsToolchain(conanfile)
        assert be.ndebug is None
        env = be.vars()
        assert "-DNDEBUG" not in env["CPPFLAGS"]


@pytest.mark.parametrize("config", [
    ("gcc", 'libstdc++', None),
    ("clang", 'libstdc++', '-stdlib=libstdc++'),
    ("clang", 'libstdc++11', '-stdlib=libstdc++'),
    ("clang", 'libc++', '-stdlib=libc++'),
    ("apple-clang", 'libstdc++', '-stdlib=libstdc++'),
    ("apple-clang", 'libstdc++11', '-stdlib=libstdc++'),
    ("apple-clang", 'libc++', '-stdlib=libc++'),
    ("sun-cc", 'libCstd', '-library=Cstd'),
    ("sun-cc", 'libstdcxx', '-library=stdcxx4'),
    ("sun-cc", 'libstlport', '-library=stlport4'),
    ("sun-cc", 'libstdc++', '-library=stdcpp'),
    ("qcc", 'libCstd', '-Y _libCstd'),
    ("qcc", 'libstdcxx', '-Y _libstdcxx'),
    ("qcc", 'libstlport', '-Y _libstlport'),
    ("qcc", 'libstdc++', '-Y _libstdc++'),
    ])
def test_libcxx(config):
    compiler, libcxx, expected_flag = config
    conanfile = ConanFileMock()
    conanfile.settings = MockSettings(
        {"build_type": "Release",
         "arch": "x86",
         "compiler": compiler,
         "compiler.libcxx": libcxx,
         "compiler.version": "7.1",
         "compiler.cppstd": "17"})
    be = AutotoolsToolchain(conanfile)
    assert be.libcxx == expected_flag
    env = be.vars()
    if expected_flag:
        assert expected_flag in env["CXXFLAGS"]


def test_cxx11_abi_define():
    conanfile = ConanFileMock()
    conanfile.settings = MockSettings(
        {"build_type": "Release",
         "arch": "x86",
         "compiler": "gcc",
         "compiler.libcxx": "libstdc++",
         "compiler.version": "7.1",
         "compiler.cppstd": "17"})
    be = AutotoolsToolchain(conanfile)
    assert be.gcc_cxx11_abi == "_GLIBCXX_USE_CXX11_ABI=0"
    env = be.vars()
    assert "-D_GLIBCXX_USE_CXX11_ABI=0" in env["CPPFLAGS"]

    conanfile.settings = MockSettings(
        {"build_type": "Release",
         "arch": "x86",
         "compiler": "gcc",
         "compiler.libcxx": "libstdc++11",
         "compiler.version": "7.1",
         "compiler.cppstd": "17"})
    be = AutotoolsToolchain(conanfile)
    env = be.vars()
    assert be.gcc_cxx11_abi is None
    assert "GLIBCXX_USE_CXX11_ABI" not in env["CPPFLAGS"]

    # Force the GLIBCXX_USE_CXX11_ABI=1 for old distros is direct def f ``gcc_cxx11_abi``
    be.gcc_cxx11_abi = "_GLIBCXX_USE_CXX11_ABI=1"
    env = be.vars()
    assert "-D_GLIBCXX_USE_CXX11_ABI=1" in env["CPPFLAGS"]

    # Also conf is possible
    conanfile.conf["tools.gnu:define_libcxx11_abi"] = True
    be = AutotoolsToolchain(conanfile)
    env = be.vars()
    assert "-D_GLIBCXX_USE_CXX11_ABI=1" in env["CPPFLAGS"]


@pytest.mark.parametrize("config", [
    ('x86_64', "-m64"),
    ('x86', "-m32")])
def test_architecture_flag(config):
    """Architecture flag is set in CXXFLAGS, CFLAGS and LDFLAGS"""
    arch, expected = config
    conanfile = ConanFileMock()
    conanfile.settings = MockSettings(
        {"build_type": "Release",
         "os": "Macos",
         "compiler": "gcc",
         "arch": arch})
    be = AutotoolsToolchain(conanfile)
    assert be.arch_flag == expected
    env = be.vars()
    assert expected in env["CXXFLAGS"]
    assert expected in env["CFLAGS"]
    assert expected in env["LDFLAGS"]
    assert "-debug" not in env["LDFLAGS"]


@pytest.mark.parametrize("compiler", ['Visual Studio', 'msvc'])
def test_build_type_flag(compiler):
    """Architecture flag is set in CXXFLAGS, CFLAGS and LDFLAGS"""
    conanfile = ConanFileMock()
    conanfile.settings = MockSettings(
        {"build_type": "Debug",
         "os": "Windows",
         "compiler": compiler,
         "arch": "x86_64"})
    be = AutotoolsToolchain(conanfile)
    assert be.build_type_flags == ["-Zi", "-Ob0", "-Od"]
    env = be.vars()
    assert "-Zi -Ob0 -Od" in env["CXXFLAGS"]
    assert "-Zi -Ob0 -Od" in env["CFLAGS"]
    assert "-Zi -Ob0 -Od" not in env["LDFLAGS"]
    assert "-debug" in env["LDFLAGS"]


def test_apple_arch_flag():
    conanfile = ConanFileMock()
    conanfile.conf.define("tools.apple:sdk_path", "/path/to/sdk")
    conanfile.settings_build = MockSettings(
        {"build_type": "Debug",
         "os": "Macos",
         "arch": "x86_64"})
    conanfile.settings = MockSettings(
        {"build_type": "Debug",
         "os": "iOS",
         "os.version": "14",
         "arch": "armv8"})
    be = AutotoolsToolchain(conanfile)
    expected = "-arch arm64"
    assert be.apple_arch_flag == expected
    env = be.vars()
    assert expected in env["CXXFLAGS"]
    assert expected in env["CFLAGS"]
    assert expected in env["LDFLAGS"]

    # Only set when crossbuilding
    conanfile = ConanFileMock()
    conanfile.settings = MockSettings(
        {"build_type": "Debug",
         "os": "iOS",
         "os.version": "14",
         "arch": "armv8"})
    be = AutotoolsToolchain(conanfile)
    assert be.apple_arch_flag is None


def test_apple_min_os_flag():
    """Even when no cross building it is adjusted because it could target a Mac version"""
    conanfile = ConanFileMock()
    conanfile.conf.define("tools.apple:sdk_path", "/path/to/sdk")
    conanfile.settings = MockSettings(
        {"build_type": "Debug",
         "os": "Macos",
         "os.version": "14",
         "arch": "armv8"})
    be = AutotoolsToolchain(conanfile)
    expected = "-mmacosx-version-min=14"
    assert be.apple_min_version_flag == expected
    env = be.vars()
    assert expected in env["CXXFLAGS"]
    assert expected in env["CFLAGS"]
    assert expected in env["LDFLAGS"]


def test_apple_isysrootflag():
    """Even when no cross building it is adjusted because it could target a Mac version"""
    conanfile = ConanFileMock()
    conanfile.conf.define("tools.apple:sdk_path", "/path/to/sdk")
    conanfile.settings_build = MockSettings(
        {"build_type": "Debug",
         "os": "Macos",
         "arch": "x86_64"})
    conanfile.settings = MockSettings(
        {"build_type": "Debug",
         "os": "iOS",
         "os.version": "14",
         "arch": "armv8"})
    be = AutotoolsToolchain(conanfile)
    expected = "-isysroot /path/to/sdk"
    assert be.apple_isysroot_flag == expected
    env = be.vars()
    assert expected in env["CXXFLAGS"]
    assert expected in env["CFLAGS"]
    assert expected in env["LDFLAGS"]

    # Only set when crossbuilding
    conanfile = ConanFileMock()
    conanfile.settings = MockSettings(
        {"build_type": "Debug",
         "os": "iOS",
         "os.version": "14",
         "arch": "armv8"})
    be = AutotoolsToolchain(conanfile)
    assert be.apple_isysroot_flag is None


def test_sysrootflag():
    """Even when no cross building it is adjusted because it could target a Mac version"""
    conanfile = ConanFileMock()
    conanfile.conf.define("tools.build:sysroot", "/path/to/sysroot")
    conanfile.settings = MockSettings(
        {"build_type": "Debug",
         "os": {"Darwin": "Macos"}.get(platform.system(), platform.system()),
         "arch": "x86_64"})
    be = AutotoolsToolchain(conanfile)
    expected = "--sysroot /path/to/sysroot"
    assert be.sysroot_flag == expected
    env = be.vars()
    assert expected in env["CXXFLAGS"]
    assert expected in env["CFLAGS"]
    assert expected in env["LDFLAGS"]


def test_custom_defines():
    conanfile = ConanFileMock()
    conanfile.conf.define("tools.apple:sdk_path", "/path/to/sdk")
    conanfile.settings = MockSettings(
        {"build_type": "RelWithDebInfo",
         "os": "iOS",
         "os.version": "14",
         "arch": "armv8"})
    be = AutotoolsToolchain(conanfile)
    be.extra_defines = ["MyDefine1", "MyDefine2"]

    assert "MyDefine1" in be.defines
    assert "MyDefine2" in be.defines
    assert "NDEBUG" in be.defines

    env = be.vars()
    assert "-DMyDefine1" in env["CPPFLAGS"]
    assert "-DMyDefine2" in env["CPPFLAGS"]
    assert "-DNDEBUG" in env["CPPFLAGS"]


def test_custom_cxxflags():
    conanfile = ConanFileMock()
    conanfile.conf.define("tools.apple:sdk_path", "/path/to/sdk")
    conanfile.settings = MockSettings(
        {"build_type": "RelWithDebInfo",
         "os": "iOS",
         "os.version": "14",
         "arch": "armv8"})
    be = AutotoolsToolchain(conanfile)
    be.extra_cxxflags = ["MyFlag1", "MyFlag2"]

    assert "MyFlag1" in be.cxxflags
    assert "MyFlag2" in be.cxxflags
    assert "-mios-version-min=14" in be.cxxflags
    assert "MyFlag" not in be.cflags
    assert "MyFlag" not in be.ldflags

    env = be.vars()
    assert "MyFlag1" in env["CXXFLAGS"]
    assert "MyFlag2" in env["CXXFLAGS"]
    assert "-mios-version-min=14" in env["CXXFLAGS"]

    assert "MyFlag" not in env["CFLAGS"]
    assert "MyFlag" not in env["LDFLAGS"]


def test_custom_cflags():
    conanfile = ConanFileMock()
    conanfile.conf.define("tools.apple:sdk_path", "/path/to/sdk")
    conanfile.settings = MockSettings(
        {"build_type": "RelWithDebInfo",
         "os": "iOS",
         "os.version": "14",
         "arch": "armv8"})
    be = AutotoolsToolchain(conanfile)
    be.extra_cflags = ["MyFlag1", "MyFlag2"]

    assert "MyFlag1" in be.cflags
    assert "MyFlag2" in be.cflags
    assert "-mios-version-min=14" in be.cflags
    assert "MyFlag" not in be.cxxflags
    assert "MyFlag" not in be.ldflags

    env = be.vars()
    assert "MyFlag1" in env["CFLAGS"]
    assert "MyFlag2" in env["CFLAGS"]
    assert "-mios-version-min=14" in env["CFLAGS"]

    assert "MyFlag" not in env["CXXFLAGS"]
    assert "MyFlag" not in env["LDFLAGS"]


def test_custom_ldflags():
    conanfile = ConanFileMock()
    conanfile.conf.define("tools.apple:sdk_path", "/path/to/sdk")
    conanfile.settings = MockSettings(
        {"build_type": "RelWithDebInfo",
         "os": "iOS",
         "os.version": "14",
         "arch": "armv8"})
    be = AutotoolsToolchain(conanfile)
    be.extra_ldflags = ["MyFlag1", "MyFlag2"]

    assert "MyFlag1" in be.ldflags
    assert "MyFlag2" in be.ldflags
    assert "-mios-version-min=14" in be.ldflags
    assert "MyFlag" not in be.cxxflags
    assert "MyFlag" not in be.cflags

    env = be.vars()
    assert "MyFlag1" in env["LDFLAGS"]
    assert "MyFlag2" in env["LDFLAGS"]
    assert "-mios-version-min=14" in env["LDFLAGS"]

    assert "MyFlag" not in env["CXXFLAGS"]
    assert "MyFlag" not in env["CFLAGS"]


def test_extra_flags_via_conf():
    conanfile = ConanFileMock()
    conanfile.conf.define("tools.build:cxxflags", ["--flag1", "--flag2"])
    conanfile.conf.define("tools.build:cflags", ["--flag3", "--flag4"])
    conanfile.conf.define("tools.build:sharedlinkflags", ["--flag5"])
    conanfile.conf.define("tools.build:exelinkflags", ["--flag6"])
    conanfile.conf.define("tools.build:defines", ["DEF1", "DEF2"])
    conanfile.settings = MockSettings(
        {"build_type": "RelWithDebInfo",
         "os": "iOS",
         "os.version": "14",
         "arch": "armv8"})
    be = AutotoolsToolchain(conanfile)
    env = be.vars()
    assert '-DNDEBUG -DDEF1 -DDEF2' in env["CPPFLAGS"]
    assert '-mios-version-min=14 --flag1 --flag2' in env["CXXFLAGS"]
    assert '-mios-version-min=14 --flag3 --flag4' in env["CFLAGS"]
    assert '-mios-version-min=14 --flag5 --flag6' in env["LDFLAGS"]



import os
import platform
import pytest
import textwrap

from conan.tools.microsoft.visual import vcvars_command
from ._base import BaseIntelTestCase

from conans.test.assets.sources import gen_function_cpp
from ..microsoft.test_msbuild import sln_file, myapp_vcxproj

conanfile_py = textwrap.dedent("""
    from conans import ConanFile, MSBuild, MSBuildToolchain

    class App(ConanFile):
        settings = 'os', 'arch', 'compiler', 'build_type'
        exports_sources = "MyProject.sln", "MyApp/MyApp.vcxproj", "MyApp/MyApp.cpp"
        generators = "msbuild"
        requires = "hello/0.1"

        def generate(self):
            tc = MSBuildToolchain(self)
            tc.generate()

        def build(self):
            msbuild = MSBuild(self)
            msbuild.build("MyProject.sln")
""")


@pytest.mark.tool_cmake
@pytest.mark.tool_msbuild
@pytest.mark.tool_icc
@pytest.mark.xfail(reason="Intel compiler not installed yet on CI")
@pytest.mark.skipif(platform.system() != "Windows", reason="msbuild requires Windows")
class MSBuildIntelTestCase(BaseIntelTestCase):
    def test_use_msbuild_toolchain(self):
        self.t.save({'profile': self.profile})
        self.t.run("new hello/0.1 -s")
        self.t.run("create . hello/0.1@ -pr:h=profile")

        app = gen_function_cpp(name="main", includes=["hello"], calls=["hello"])

        # Prepare the actual consumer package
        self.t.save({"conanfile.py": conanfile_py,
                     "MyProject.sln": sln_file,
                     "MyApp/MyApp.vcxproj": myapp_vcxproj,
                     "MyApp/MyApp.cpp": app,
                     'profile': self.profile},
                    clean_first=True)

        # Build in the cache
        self.t.run("install . -pr:h=profile -if=conan")

        self.assertIn("conanfile.py: MSBuildToolchain created conan_toolchain_release_x64.props",
                      self.t.out)

        self.t.run("build . -if=conan")
        self.assertIn("Visual Studio 2017", self.t.out)
        self.assertIn("[vcvarsall.bat] Environment initialized for: 'x64'", self.t.out)

        exe = "x64\\Release\\MyApp.exe"
        self.t.run_command(exe)
        self.assertIn("main __INTEL_COMPILER1910", self.t.out)

        vcvars = vcvars_command(version="15", architecture="x64")
        dumpbind_cmd = '%s && dumpbin /dependents "%s"' % (vcvars, exe)
        self.t.run_command(dumpbind_cmd)
        self.assertIn("KERNEL32.dll", self.t.out)

        # Build locally
        os.unlink(os.path.join(self.t.current_folder, exe))

        cmd = vcvars + ' && msbuild "MyProject.sln" /p:Configuration=Release ' \
                       '/p:Platform=x64 /p:PlatformToolset="Intel C++ Compiler 19.1"'

        self.t.run_command(cmd)
        self.assertIn("Visual Studio 2017", self.t.out)
        self.assertIn("[vcvarsall.bat] Environment initialized for: 'x64'", self.t.out)

        self.t.run_command(exe)
        self.assertIn("main __INTEL_COMPILER1910", self.t.out)

        self.t.run_command(dumpbind_cmd)
        self.assertIn("KERNEL32.dll", self.t.out)



import os
import re
import textwrap
import unittest

from conans.model.graph_info import GRAPH_INFO_FILE
from conans.model.graph_lock import LOCKFILE
from conans.model.ref import ConanFileReference, PackageReference
from conans.paths import (BUILD_INFO, BUILD_INFO_CMAKE, BUILD_INFO_GCC, BUILD_INFO_VISUAL_STUDIO,
                          BUILD_INFO_XCODE, CONANFILE_TXT, CONANINFO)
from conans.test.assets.genconanfile import GenConanfile
from conans.test.utils.tools import TestClient


class VSXCodeGeneratorsTest(unittest.TestCase):

    def test_frameworks_no_compiler(self):
        client = TestClient()
        client.save({"conanfile.py": textwrap.dedent("""
            from conans import ConanFile
            class Pkg(ConanFile):
                def package_info(self):
                    self.cpp_info.frameworks = ["CoreAudio"]
            """)})
        client.run("export . Hello/0.1@lasote/stable")

        client.save({"conanfile.py": GenConanfile().with_requires("Hello/0.1@lasote/stable").with_generator("xcode")})
        client.run('install . --build missing')
        xcode = client.load(BUILD_INFO_XCODE)
        self.assertIn('OTHER_LDFLAGS = $(inherited)    -framework CoreAudio', xcode)

    def test_generators(self):
        ref = ConanFileReference.loads("Hello/0.1@lasote/stable")
        client = TestClient()
        client.save({"conanfile.py": textwrap.dedent("""
            from conans import ConanFile
            import os
            class Pkg(ConanFile):
                def package(self):
                    os.makedirs(os.path.join(self.package_folder, "lib"))
                    os.makedirs(os.path.join(self.package_folder, "include"))
                def package_info(self):
                    self.cpp_info.libs = ["hello"]
                    self.cpp_info.cxxflags = ["-some_cxx_compiler_flag"]
                    self.cpp_info.cflags = ["-some_c_compiler_flag"]
                    self.cpp_info.system_libs = ["system_lib1"]
            """)})
        client.run("export . Hello/0.1@lasote/stable")
        conanfile_txt = textwrap.dedent('''
            [requires]
            Hello/0.1@lasote/stable # My req comment
            [generators]
            gcc # I need this generator for..
            cmake
            visual_studio
            xcode
            ''')
        client.save({"conanfile.txt": conanfile_txt}, clean_first=True)

        # Install requirements
        client.run('install . --build missing')
        current_files = os.listdir(client.current_folder)
        for f in [CONANFILE_TXT, BUILD_INFO_GCC, BUILD_INFO_CMAKE, BUILD_INFO_VISUAL_STUDIO,
                  BUILD_INFO, BUILD_INFO_XCODE, CONANINFO, GRAPH_INFO_FILE, LOCKFILE]:
            assert f in current_files

        cmake = client.load(BUILD_INFO_CMAKE)
        gcc = client.load(BUILD_INFO_GCC)

        self.assertIn("CONAN_INCLUDE_DIRS", cmake)
        self.assertIn("CONAN_LIB_DIRS", cmake)
        self.assertIn("CONAN_LIBS", cmake)

        self.assertIn("CONAN_INCLUDE_DIRS", cmake)
        self.assertIn("CONAN_LIB_DIRS", cmake)
        self.assertIn("/data/Hello/0.1/lasote/stable/package", cmake)

        self.assertIn("-L", gcc)
        self.assertIn("-l", gcc)
        self.assertIn("-I", gcc)

        self.assertIn("/data/Hello/0.1/lasote/stable/package", gcc)

        # CHECK VISUAL STUDIO GENERATOR

        from xml.dom import minidom
        xmldoc = minidom.parse(os.path.join(client.current_folder, BUILD_INFO_VISUAL_STUDIO))
        definition_group = xmldoc.getElementsByTagName('ItemDefinitionGroup')[0]
        _ = definition_group.getElementsByTagName("ClCompile")[0]
        linker = definition_group.getElementsByTagName("Link")[0]

        def element_content(node):
            return node.firstChild.data if node.firstChild else ""

        include_dirs = element_content(xmldoc.getElementsByTagName("ConanIncludeDirectories")[0])
        definitions = element_content(xmldoc.getElementsByTagName("ConanPreprocessorDefinitions")[0])
        lib_dirs = element_content(xmldoc.getElementsByTagName("ConanLibraryDirectories")[0])
        libs = element_content(linker.getElementsByTagName("AdditionalDependencies")[0])
        system_libs = element_content(linker.getElementsByTagName("AdditionalDependencies")[1])

        package_id = os.listdir(client.cache.package_layout(ref).packages())[0]
        pref = PackageReference(ref, package_id)
        package_path = client.cache.package_layout(pref.ref).package(pref)

        replaced_path = re.sub(os.getenv("USERPROFILE", "not user profile").replace("\\", "\\\\"),
                               "$(USERPROFILE)", package_path, flags=re.I)
        expected_lib_dirs = os.path.join(replaced_path, "lib")
        expected_include_dirs = os.path.join(replaced_path, "include")

        self.assertIn(expected_lib_dirs, lib_dirs)
        self.assertEqual("$(ConanLibraries)%(AdditionalDependencies)", libs)
        self.assertEqual("$(ConanSystemDeps)%(AdditionalDependencies)", system_libs)
        self.assertEqual("", definitions)
        self.assertIn(expected_include_dirs, include_dirs)

        # CHECK XCODE GENERATOR
        xcode = client.load(BUILD_INFO_XCODE)

        expected_c_flags = '-some_c_compiler_flag'
        expected_cpp_flags = '-some_cxx_compiler_flag'
        expected_lib_dirs = os.path.join(package_path, "lib").replace("\\", "/")
        expected_include_dirs = os.path.join(package_path, "include").replace("\\", "/")

        self.assertIn('LIBRARY_SEARCH_PATHS = $(inherited) "%s"' % expected_lib_dirs, xcode)
        self.assertIn('HEADER_SEARCH_PATHS = $(inherited) "%s"' % expected_include_dirs, xcode)
        self.assertIn("GCC_PREPROCESSOR_DEFINITIONS = $(inherited)", xcode)
        self.assertIn('OTHER_CFLAGS = $(inherited) %s' % expected_c_flags, xcode)
        self.assertIn('OTHER_CPLUSPLUSFLAGS = $(inherited) %s' % expected_cpp_flags, xcode)
        self.assertIn('FRAMEWORK_SEARCH_PATHS = $(inherited) "%s"' % package_path.replace("\\", "/"),
                      xcode)
        self.assertIn('OTHER_LDFLAGS = $(inherited)  -lhello -lsystem_lib1',
                      xcode)



import os
import time
import unittest

import pytest

from conans.test.utils.tools import TestClient


@pytest.mark.slow
class ConanTestTest(unittest.TestCase):

    @pytest.mark.tool_cmake
    def test_conan_test(self):
        client = TestClient()
        client.run("new hello/0.1 -m=cmake_lib")

        client.run("create . lasote/stable -tf=None")
        time.sleep(1)  # Try to avoid windows errors in CI  (Cannot change permissions)
        client.run("test test_package hello/0.1@lasote/stable -s build_type=Release")
        self.assertIn('hello/0.1: Hello World Release!', client.out)

        self.assertNotIn("WARN: conanbuildinfo.txt file not found", client.out)
        self.assertNotIn("WARN: conanenv.txt file not found", client.out)

        client.run("test test_package hello/0.1@lasote/stable -s hello:build_type=Debug "
                   "--build missing")
        self.assertIn('hello/0.1: Hello World Debug!', client.out)
        assert os.path.exists(os.path.join(client.current_folder, "test_package",
                                           "build", "generators", "conaninfo.txt"))



from conans.client.tools import environment_append
from conans.test.utils.tools import GenConanfile, TestClient
from conans.util.files import save


def test_recipe_modes():
    configs = []
    mode = "semver_mode"
    configs.append((mode, "liba/1.1.1@user/testing", "8ecbf93ba63522ffb32573610c80ab4dcb399b52"))
    configs.append((mode, "liba/1.1.2@user/testing", "8ecbf93ba63522ffb32573610c80ab4dcb399b52"))
    configs.append((mode, "liba/1.2.1@other/stable", "8ecbf93ba63522ffb32573610c80ab4dcb399b52"))
    mode = "patch_mode"
    configs.append((mode, "liba/1.1.1@user/testing", "bd664570d5174c601d5d417bc19257c4dba48f2e"))
    configs.append((mode, "liba/1.1.2@user/testing", "fb1f766173191d44b67156c6b9ac667422e99286"))
    configs.append((mode, "liba/1.1.1@other/stable", "bd664570d5174c601d5d417bc19257c4dba48f2e"))
    mode = "full_recipe_mode"
    configs.append((mode, "liba/1.1.1@user/testing", "9cbe703e1dee73a2a6807f71d8551c5f1e1b08fd"))
    configs.append((mode, "liba/1.1.2@user/testing", "42a9ff9024adabbd54849331cf01be7d95139948"))
    configs.append((mode, "liba/1.1.1@user/stable", "b41d6c026473cffed4abded4b0eaa453497be1d2"))

    client = TestClient()
    # TODO: These 2 little simplifications can reduce test time by 30-40%, to do in test framework
    save(client.cache.settings_path, "")
    save(client.cache.default_profile_path, "")

    def _assert_recipe_mode(liba_ref, package_id_arg):
        client.save({"liba/conanfile.py": GenConanfile("liba"),
                     "libb/conanfile.py": GenConanfile("libb", "0.1").with_require(liba_ref),
                     "app/conanfile.py": GenConanfile("app", "0.1").with_require("libb/0.1")})
        client.run("create liba {}".format(liba_ref))
        client.run("create libb")
        client.run("create app")

        assert "{}:5ab84d6acfe1f23c4fae0ab88f26e3a396351ac9 - Cache".format(liba_ref) in client.out
        assert "libb/0.1:{} - Cache".format(package_id_arg) in client.out

    for package_id_mode, ref, package_id in configs:
        client.run("config set general.default_package_id_mode={}".format(package_id_mode))
        _assert_recipe_mode(ref, package_id)

    for package_id_mode, ref, package_id in configs:
        with environment_append({"CONAN_DEFAULT_PACKAGE_ID_MODE": package_id_mode}):
            _assert_recipe_mode(ref, package_id)


def test_package_revision_mode():
    client = TestClient()
    # TODO: These 2 little simplifications can reduce test time by 30-40%, to do in test framework
    save(client.cache.settings_path, "")
    save(client.cache.default_profile_path, "")

    client.run("config set general.default_package_id_mode=package_revision_mode")

    client.save({"liba/conanfile.py": GenConanfile("liba", "0.1.1"),
                 "libb/conanfile.py": GenConanfile("libb", "0.1").with_require("liba/0.1.1"),
                 "app/conanfile.py": GenConanfile("app", "0.1").with_require("libb/0.1")})

    client.run("create liba")
    client.run("create libb")
    client.run("create app")

    assert "liba/0.1.1:5ab84d6acfe1f23c4fae0ab88f26e3a396351ac9 - Cache" in client.out
    assert "libb/0.1:f2431acdad185a6626b90ee1dffb06445f887cbb - Cache" in client.out



import unittest

from mock import Mock

from conans import Settings
from conans.client.generators import CMakeFindPackageMultiGenerator
from conans.model.build_info import CppInfo
from conans.model.conan_file import ConanFile
from conans.model.env_info import EnvValues
from conans.model.ref import ConanFileReference
from conans.test.unittests.client.generators.cmake_test import _MockSettings


class CMakeFindPackageMultiTest(unittest.TestCase):

    def test_cmake_find_package_multi_version(self):
        # https://github.com/conan-io/conan/issues/6908
        settings_mock = _MockSettings(build_type="Debug")
        conanfile = ConanFile(Mock(), None)
        conanfile.initialize(settings_mock, EnvValues())
        ref = ConanFileReference.loads("my_pkg/0.1@user/stable")
        cpp_info = CppInfo(ref.name, "")
        cpp_info.version = ref.version
        cpp_info.debug.libs = ["mylib"]
        conanfile.deps_cpp_info.add(ref.name, cpp_info)

        generator = CMakeFindPackageMultiGenerator(conanfile)
        content = generator.content
        config_version = content["my_pkg-config-version.cmake"]
        self.assertIn('set(PACKAGE_VERSION "0.1")', config_version)


def test_cmake_find_package_multi_links_flags():
    # https://github.com/conan-io/conan/issues/8703
    conanfile = ConanFile(Mock(), None)
    conanfile.settings = "os", "compiler", "build_type", "arch"
    conanfile.initialize(Settings({"os": ["Windows"],
                                   "compiler": ["gcc"],
                                   "build_type": ["Release"],
                                   "arch": ["x86"]}), EnvValues())
    conanfile.settings.build_type = "Release"
    conanfile.settings.arch = "x86"

    cpp_info = CppInfo("mypkg", "dummy_root_folder1")
    # https://github.com/conan-io/conan/issues/8811 regression, fix with explicit - instead of /
    cpp_info.sharedlinkflags = ["-NODEFAULTLIB", "-OTHERFLAG"]
    cpp_info.exelinkflags = ["-OPT:NOICF"]
    conanfile.deps_cpp_info.add("mypkg", cpp_info)

    gen = CMakeFindPackageMultiGenerator(conanfile)
    files = gen.content
    d = files["mypkgTarget-release.cmake"]
    assert "$<$<STREQUAL:$<TARGET_PROPERTY:TYPE>,EXECUTABLE>:-OPT:NOICF>" in d
    assert "$<$<STREQUAL:$<TARGET_PROPERTY:TYPE>,MODULE_LIBRARY>:-NODEFAULTLIB;-OTHERFLAG>" in d
    assert "$<$<STREQUAL:$<TARGET_PROPERTY:TYPE>,SHARED_LIBRARY>:-NODEFAULTLIB;-OTHERFLAG>" in d



import textwrap
import unittest

from conans.test.utils.tools import TestClient


class ConanfileErrorsTest(unittest.TestCase):

    def test_copy_error(self):
        client = TestClient()
        conanfile = textwrap.dedent('''
            from conans import ConanFile

            class HelloConan(ConanFile):
                name = "Hello"
                version = "0.1"
                exports = "*"
                def package(self):
                    self.copy2("*.h", dst="include", src=["include","platform"])
            ''')
        files = {"conanfile.py": conanfile, "test.txt": "Hello world"}
        client.save(files)
        client.run("export . lasote/stable")
        client.run("install Hello/0.1@lasote/stable --build", assert_error=True)
        self.assertIn("Hello/0.1@lasote/stable: Error in package() method, line 9", client.out)
        self.assertIn('self.copy2("*.h", dst="include", src=["include","platform"]', client.out)
        self.assertIn("'HelloConan' object has no attribute 'copy2'", client.out)

    def test_copy_error2(self):
        client = TestClient()
        conanfile = textwrap.dedent('''
            from conans import ConanFile

            class HelloConan(ConanFile):
                name = "Hello"
                version = "0.1"
                exports = "*"
                def package(self):
                    self.copy("*.h", dst="include", src=["include","platform"])
            ''')
        files = {"conanfile.py": conanfile, "test.txt": "Hello world"}
        client.save(files)
        client.run("export . lasote/stable")
        client.run("install Hello/0.1@lasote/stable --build", assert_error=True)
        self.assertIn("Hello/0.1@lasote/stable: Error in package() method, line 9", client.out)
        self.assertIn('self.copy("*.h", dst="include", src=["include","platform"]', client.out)
        # It results that the error is different in different Python2/3 and OSs
        # self.assertIn("'list' object has no attribute 'replace'", client.out)

    def test_package_info_error(self):
        client = TestClient()
        conanfile = textwrap.dedent('''
            from conans import ConanFile

            class HelloConan(ConanFile):
                name = "Hello"
                version = "0.1"
                exports = "*"
                def package_info(self):
                    self.copy2()
            ''')
        files = {"conanfile.py": conanfile, "test.txt": "Hello world"}
        client.save(files)
        client.run("export . lasote/stable")
        client.run("install Hello/0.1@lasote/stable --build", assert_error=True)
        self.assertIn("Hello/0.1@lasote/stable: Error in package_info() method, line 9", client.out)
        self.assertIn('self.copy2()', client.out)
        self.assertIn("'HelloConan' object has no attribute 'copy2'", client.out)

    def test_config_error(self):
        client = TestClient()
        conanfile = textwrap.dedent('''
            from conans import ConanFile

            class HelloConan(ConanFile):
                name = "Hello"
                version = "0.1"
                exports = "*"
                def configure(self):
                    self.copy2()
            ''')
        files = {"conanfile.py": conanfile, "test.txt": "Hello world"}
        client.save(files)
        client.run("export . lasote/stable")
        client.run("install Hello/0.1@lasote/stable --build", assert_error=True)

        self.assertIn("ERROR: Hello/0.1@lasote/stable: Error in configure() method, line 9",
                      client.out)
        self.assertIn("self.copy2()", client.out)
        self.assertIn("AttributeError: 'HelloConan' object has no attribute 'copy2'""", client.out)

    def test_source_error(self):
        client = TestClient()
        conanfile = textwrap.dedent('''
            from conans import ConanFile

            class HelloConan(ConanFile):
                name = "Hello"
                version = "0.1"
                exports = "*"
                def source(self):
                    self.copy2()
            ''')
        files = {"conanfile.py": conanfile, "test.txt": "Hello world"}
        client.save(files)
        client.run("export . lasote/stable")
        client.run("install Hello/0.1@lasote/stable --build", assert_error=True)
        self.assertIn("Hello/0.1@lasote/stable: Error in source() method, line 9", client.out)
        self.assertIn('self.copy2()', client.out)
        self.assertIn("'HelloConan' object has no attribute 'copy2'", client.out)

    def test_duplicate_requires(self):
        client = TestClient()
        conanfile = textwrap.dedent('''
            [requires]
            foo/0.1@user/testing
            foo/0.2@user/testing
            ''')
        files = {"conanfile.txt": conanfile}
        client.save(files)
        client.run("install . --build", assert_error=True)
        self.assertIn("ERROR: Duplicated requirement", client.out)

    def test_duplicate_requires_py(self):
        client = TestClient()
        conanfile = textwrap.dedent('''
            from conans import ConanFile

            class HelloConan(ConanFile):
                name = "Hello"
                version = "0.1"
                requires = "foo/0.1@user/testing", "foo/0.2@user/testing"
            ''')
        files = {"conanfile.py": conanfile}
        client.save(files)
        client.run("install . --build", assert_error=True)
        self.assertIn("Error while initializing requirements. Duplicated requirement", client.out)



import os
import textwrap
import platform

import pytest

from conans.client.tools.apple import to_apple_arch
from conans.test.assets.autotools import gen_makefile
from conans.test.assets.sources import gen_function_h, gen_function_cpp
from conans.test.utils.tools import TestClient

makefile = gen_makefile(apps=["app"], libs=["hello"])

conanfile_py = textwrap.dedent("""
    from conans import ConanFile, tools
    from conan.tools.gnu import Autotools

    class App(ConanFile):
        settings = "os", "arch", "compiler", "build_type"
        options = {"shared": [True, False], "fPIC": [True, False]}
        default_options = {"shared": False, "fPIC": True}
        generators = "AutotoolsToolchain"

        def config_options(self):
            if self.settings.os == "Windows":
                del self.options.fPIC

        def build(self):
            env_build = Autotools(self)
            env_build.make()
    """)


@pytest.mark.skipif(platform.system() != "Darwin", reason="Only OSX")
@pytest.mark.parametrize("config", [("x86_64", "Macos", "10.14"),
                                    ("armv8", "iOS", "10.0"),
                                    ("armv7", "iOS", "10.0"),
                                    ("x86", "iOS", "10.0"),
                                    ("x86_64", "iOS", "10.0"),
                                    ("armv8", "Macos", "10.14")  # M1
                                    ])
def test_makefile_arch(config):
    arch, os_, os_version = config
    profile = textwrap.dedent("""
                include(default)
                [settings]
                os = {os}
                os.version = {os_version}
                arch = {arch}
                """).format(os=os_, arch=arch, os_version=os_version)

    t = TestClient()
    hello_h = gen_function_h(name="hello")
    hello_cpp = gen_function_cpp(name="hello")
    main_cpp = gen_function_cpp(name="main", includes=["hello"], calls=["hello"])

    t.save({"Makefile": makefile,
            "hello.h": hello_h,
            "hello.cpp": hello_cpp,
            "app.cpp": main_cpp,
            "conanfile.py": conanfile_py,
            "profile": profile})

    t.run("install . --profile:host=profile --profile:build=default")
    t.run("build .")

    libhello = os.path.join(t.current_folder, "libhello.a")
    app = os.path.join(t.current_folder, "app")
    assert os.path.isfile(libhello)
    assert os.path.isfile(app)

    expected_arch = to_apple_arch(arch)

    t.run_command('lipo -info "%s"' % libhello)
    assert "architecture: %s" % expected_arch in t.out

    t.run_command('lipo -info "%s"' % app)
    assert "architecture: %s" % expected_arch in t.out


@pytest.mark.skipif(platform.system() != "Darwin", reason="Only OSX")
@pytest.mark.parametrize("arch", ["x86_64", "armv8"])
def test_catalyst(arch):
    profile = textwrap.dedent("""
        include(default)
        [settings]
        os = Macos
        os.version = 13.0
        os.sdk = macosx
        os.subsystem = catalyst
        os.subsystem.ios_version = 13.1
        arch = {arch}
        """).format(arch=arch)

    t = TestClient()
    hello_h = gen_function_h(name="hello")
    hello_cpp = gen_function_cpp(name="hello")
    main_cpp = textwrap.dedent("""
        #include "hello.h"
        #include <TargetConditionals.h>
        #include <iostream>

        int main()
        {
        #if TARGET_OS_MACCATALYST
            std::cout << "running catalyst " << __IPHONE_OS_VERSION_MIN_REQUIRED << std::endl;
        #else
            #error "not building for Apple Catalyst"
        #endif
        }
        """)

    t.save({"Makefile": makefile,
            "hello.h": hello_h,
            "hello.cpp": hello_cpp,
            "app.cpp": main_cpp,
            "conanfile.py": conanfile_py,
            "profile": profile})

    t.run("install . --profile:host=profile --profile:build=default")
    t.run("build .")

    libhello = os.path.join(t.current_folder, "libhello.a")
    app = os.path.join(t.current_folder, "app")
    assert os.path.isfile(libhello)
    assert os.path.isfile(app)

    expected_arch = to_apple_arch(arch)

    t.run_command('lipo -info "%s"' % libhello)
    assert "architecture: %s" % expected_arch in t.out

    t.run_command('lipo -info "%s"' % app)
    assert "architecture: %s" % expected_arch in t.out

    if arch == "x86_64":
        t.run_command('"%s"' % app)
        assert "running catalyst 130100" in t.out



import os
import textwrap

import pytest

from conans.test.assets.cmake import gen_cmakelists
from conans.test.assets.sources import gen_function_cpp, gen_function_h
from conans.test.utils.tools import TestClient


@pytest.mark.tool_cmake
def test_file_api():
    """
    simple library providing 3 targets:
    - decoder
    - encoder
    - transcoder (requires decoder and encoder)
    generates the following targets:
    - triunfo::decoder
    - triunfo::encoder
    - triunfo::transcoder (depends on triunfo::decoder and triunfo::encoder)
    consumer uses find_package(triunfo COMPONENTS <component>)
    """
    client = TestClient()

    conanfile = textwrap.dedent("""
        from conans import ConanFile
        from conan.tools.cmake import CMake, CMakeFileAPI
        from conan.tools.files import CppPackage

        class Triunfo(ConanFile):
            name = "triunfo"
            version = "1.0"
            settings = "os", "compiler", "arch", "build_type"
            exports_sources = "*CMakeLists.txt", "*.cpp", "*.h"
            generators = "CMakeToolchain"

            def build(self):
                file_api = CMakeFileAPI(self)
                file_api.query(CMakeFileAPI.CODEMODELV2)
                cmake = CMake(self)
                cmake.configure()
                reply = file_api.reply(CMakeFileAPI.CODEMODELV2)
                package = reply.to_conan_package()
                package.save()
                cmake.build()

            def package(self):
                cmake = CMake(self)
                cmake.install()
                self.copy(CppPackage.DEFAULT_FILENAME)

            def package_info(self):
                cpp_package = CppPackage.load(CppPackage.DEFAULT_FILENAME)
                cpp_package.package_info(self)
    """)

    decoder_cpp = gen_function_cpp(name="decoder", includes=["decoder"])
    encoder_cpp = gen_function_cpp(name="encoder", includes=["encoder"])
    transcoder_cpp = gen_function_cpp(name="transcoder", calls=["decoder", "encoder"],
                                      includes=["transcoder", "../decoder/decoder", "../encoder/encoder"])
    decoder_h = gen_function_h(name="decoder")
    encoder_h = gen_function_h(name="encoder")
    transcoder_h = gen_function_h(name="transcoder")
    decoder_cmake = gen_cmakelists(libname="decoder", libsources=["decoder.cpp"], install=True,
                                   public_header="decoder.h")
    encoder_cmake = gen_cmakelists(libname="encoder", libsources=["encoder.cpp"], install=True,
                                   public_header="encoder.h")
    transcoder_cmake = gen_cmakelists(libname="transcoder", libsources=["transcoder.cpp"], install=True,
                                      public_header="transcoder.h", deps=["decoder", "encoder"])
    common_cmake = textwrap.dedent("""
        cmake_minimum_required(VERSION 2.8)
        project(triunfo)
        add_subdirectory(decoder)
        add_subdirectory(encoder)
        add_subdirectory(transcoder)
    """)

    client.save({"conanfile.py": conanfile,
                 os.path.join("decoder", "decoder.cpp"): decoder_cpp,
                 os.path.join("encoder", "encoder.cpp"): encoder_cpp,
                 os.path.join("transcoder", "transcoder.cpp"): transcoder_cpp,
                 os.path.join("decoder", "decoder.h"): decoder_h,
                 os.path.join("encoder", "encoder.h"): encoder_h,
                 os.path.join("transcoder", "transcoder.h"): transcoder_h,
                 os.path.join("decoder", "CMakeLists.txt"): decoder_cmake,
                 os.path.join("encoder", "CMakeLists.txt"): encoder_cmake,
                 os.path.join("transcoder", "CMakeLists.txt"): transcoder_cmake,
                 "CMakeLists.txt": common_cmake,
                 })
    client.run("create .")

    conanfile = textwrap.dedent("""
        from conans import ConanFile
        from conan.tools.cmake import CMake, CMakeFileAPI
        from conan.tools.files import CppPackage

        class Elogio(ConanFile):
            name = "elogio"
            version = "1.0"
            requires = "triunfo/1.0"
            settings = "os", "compiler", "arch", "build_type"
            exports_sources = "*CMakeLists.txt", "*.cpp", "*.h"
            generators = "CMakeDeps", "CMakeToolchain"

            def build(self):
                file_api = CMakeFileAPI(self)
                file_api.query(CMakeFileAPI.CODEMODELV2)
                cmake = CMake(self)
                cmake.configure()
                reply = file_api.reply(CMakeFileAPI.CODEMODELV2)
                package = reply.to_conan_package()
                package.save()
                cmake.build()
    """)

    use_decoder_cpp = gen_function_cpp(name="main", includes=["decoder"], calls=["decoder"])
    use_encoder_cpp = gen_function_cpp(name="main", includes=["encoder"], calls=["encoder"])
    use_transcoder_cpp = gen_function_cpp(name="main", includes=["transcoder"], calls=["transcoder"])
    use_decoder_cmake = gen_cmakelists(appname="use_decoder", appsources=["use_decoder.cpp"],
                                       find_package={"triunfo": "decoder"})
    use_encoder_cmake = gen_cmakelists(appname="use_encoder", appsources=["use_encoder.cpp"],
                                       find_package={"triunfo": "encoder"})
    use_transcoder_cmake = gen_cmakelists(appname="use_transcoder", appsources=["use_transcoder.cpp"],
                                          find_package={"triunfo": "transcoder"})
    common_cmake = textwrap.dedent("""
        cmake_minimum_required(VERSION 2.8)
        project(elogio)
        add_subdirectory(use_decoder)
        add_subdirectory(use_encoder)
        add_subdirectory(use_transcoder)
    """)

    client.save({"conanfile.py": conanfile,
                 os.path.join("use_decoder", "use_decoder.cpp"): use_decoder_cpp,
                 os.path.join("use_encoder", "use_encoder.cpp"): use_encoder_cpp,
                 os.path.join("use_transcoder", "use_transcoder.cpp"): use_transcoder_cpp,
                 os.path.join("use_decoder", "CMakeLists.txt"): use_decoder_cmake,
                 os.path.join("use_encoder", "CMakeLists.txt"): use_encoder_cmake,
                 os.path.join("use_transcoder", "CMakeLists.txt"): use_transcoder_cmake,
                 "CMakeLists.txt": common_cmake,
                 }, clean_first=True)

    client.run("install .")
    client.run("build .")



import json
import unittest

import pytest
from parameterized import parameterized

from conans.test.utils.tools import TestClient, GenConanfile
from conans.util.env_reader import get_env


class BuildOrderTest(unittest.TestCase):

    def test_single_consumer(self):
        # https://github.com/conan-io/conan/issues/5727
        client = TestClient()
        client.save({"conanfile.py": GenConanfile("test4", "0.1")})
        client.run("lock create conanfile.py --build --lockfile-out=conan.lock")
        client.run("lock build-order conan.lock --json=bo.json")
        jsonbo = json.loads(client.load("bo.json"))
        self.assertEqual([], jsonbo)

    def test_base_graph(self):
        client = TestClient()
        client.save({"conanfile.py": GenConanfile("test4", "0.1")})
        client.run("lock create conanfile.py --base --lockfile-out=conan.lock")
        client.run("lock build-order conan.lock --json=bo.json", assert_error=True)
        self.assertIn("Lockfiles with --base do not contain profile information, "
                      "cannot be used. Create a full lockfile", client.out)

    @parameterized.expand([(True,), (False,)])
    def test_build_not_locked(self, export):
        # https://github.com/conan-io/conan/issues/5727
        client = TestClient()
        client.save({"conanfile.py": GenConanfile("test4", "0.1")})
        if export:
            client.run("export .")
            client.run("lock create --reference=test4/0.1@ --lockfile-out=conan.lock")
        else:
            client.run("create .")
            client.run("lock create --reference=test4/0.1@ --build=test4 --lockfile-out=conan.lock")
        if client.cache.config.revisions_enabled:
            ref = "test4/0.1#f876ec9ea0f44cb7adb1588e431b391a"
            prev = "92cf292e73488c3527dab5f5ba81b947"
            build_order = [[["test4/0.1@#f876ec9ea0f44cb7adb1588e431b391a",
                             "5ab84d6acfe1f23c4fae0ab88f26e3a396351ac9", "host", "1"]]]
        else:
            ref = "test4/0.1"
            prev = "0"
            build_order = [[["test4/0.1@", "5ab84d6acfe1f23c4fae0ab88f26e3a396351ac9", "host", "1"]]]
        locked = json.loads(client.load("conan.lock"))["graph_lock"]["nodes"]
        test4 = locked["1"]
        self.assertEqual(test4["ref"], ref)
        self.assertEqual(test4["package_id"], "5ab84d6acfe1f23c4fae0ab88f26e3a396351ac9")
        self.assertEqual(test4.get("prev"), None)  # PREV is not defined yet, only exported
        client.run("lock build-order conan.lock --json=bo.json")
        jsonbo = json.loads(client.load("bo.json"))
        self.assertEqual(build_order, jsonbo)
        client.run("install test4/0.1@ --lockfile=conan.lock --lockfile-out=conan.lock --build")
        self.assertIn("test4/0.1:5ab84d6acfe1f23c4fae0ab88f26e3a396351ac9 - Build", client.out)
        locked = json.loads(client.load("conan.lock"))["graph_lock"]["nodes"]
        test4 = locked["1"]
        self.assertEqual(test4["ref"], ref)
        self.assertEqual(test4["package_id"], "5ab84d6acfe1f23c4fae0ab88f26e3a396351ac9")
        self.assertEqual(test4["prev"], prev)

        # New build order, nothing else to do
        client.run("lock build-order conan.lock --json=bo.json")
        jsonbo = json.loads(client.load("bo.json"))
        self.assertEqual([], jsonbo)

    def test_build_locked_error(self):
        client = TestClient()
        client.save({"conanfile.py": GenConanfile("test4", "0.1")})
        client.run("create .")
        client.run("lock create --reference=test4/0.1@ --lockfile-out=conan.lock")
        locked = json.loads(client.load("conan.lock"))["graph_lock"]["nodes"]
        test4 = locked["1"]
        if client.cache.config.revisions_enabled:
            ref = "test4/0.1#f876ec9ea0f44cb7adb1588e431b391a"
            prev = "92cf292e73488c3527dab5f5ba81b947"
        else:
            ref = "test4/0.1"
            prev = "0"
        self.assertEqual(test4["ref"], ref)
        self.assertEqual(test4["package_id"], "5ab84d6acfe1f23c4fae0ab88f26e3a396351ac9")
        self.assertEqual(test4["prev"], prev)
        client.run("lock build-order conan.lock --json=bo.json")
        jsonbo = json.loads(client.load("bo.json"))
        self.assertEqual([], jsonbo)
        # if we try to build anyway, error
        client.run("install test4/0.1@ --lockfile=conan.lock --build", assert_error=True)
        rev = "#f876ec9ea0f44cb7adb1588e431b391a" if client.cache.config.revisions_enabled else ""
        self.assertIn("Cannot build 'test4/0.1{}' because it is "
                      "already locked in the input lockfile".format(rev), client.out)

    @parameterized.expand([(True,), (False,)])
    def test_transitive_build_not_locked(self, export):
        # https://github.com/conan-io/conan/issues/5727
        client = TestClient()
        client.save({"dep/conanfile.py": GenConanfile(),
                     "pkg/conanfile.py": GenConanfile().with_require("dep/0.1"),
                     "app/conanfile.py": GenConanfile().with_require("pkg/0.1")})
        if export:
            client.run("export dep dep/0.1@")
            client.run("export pkg pkg/0.1@")
            client.run("export app app/0.1@")
            client.run("lock create --reference=app/0.1@ --lockfile-out=conan.lock")
        else:
            client.run("create dep dep/0.1@")
            client.run("create pkg pkg/0.1@")
            client.run("create app app/0.1@")
            client.run("lock create --reference=app/0.1@ --build --lockfile-out=conan.lock")

        locked = json.loads(client.load("conan.lock"))["graph_lock"]["nodes"]

        if client.cache.config.revisions_enabled:
            build_order = [[["dep/0.1@#f3367e0e7d170aa12abccb175fee5f97",
                             "5ab84d6acfe1f23c4fae0ab88f26e3a396351ac9", "host", "3"]],
                           [["pkg/0.1@#447b56f0334b7e2a28aa86e218c8b3bd",
                             "0b3845ce7fd8c0b4e46566097797bd872cb5bcf6", "host", "2"]],
                           [["app/0.1@#5e0af887c3e9391c872773734ccd2ca0",
                             "745ccd40fd696b66b0cb160fd5251a533563bbb4", "host", "1"]]]
            prev_dep = "83c38d3b4e5f1b8450434436eec31b00"
            prev_pkg = "bcde0c25612a6d296cf2cab2c264054d"
            prev_app = "9f30558ce471f676e3e06b633aabcf99"
        else:
            build_order = [[["dep/0.1@",
                             "5ab84d6acfe1f23c4fae0ab88f26e3a396351ac9", "host", "3"]],
                           [["pkg/0.1@",
                             "0b3845ce7fd8c0b4e46566097797bd872cb5bcf6", "host", "2"]],
                           [["app/0.1@",
                             "745ccd40fd696b66b0cb160fd5251a533563bbb4", "host", "1"]]]
            prev_dep = "0"
            prev_pkg = "0"
            prev_app = "0"

        for level in build_order:
            for item in level:
                ref, package_id, _, lockid = item
                ref = ref.replace("@", "")
                node = locked[lockid]
                self.assertEqual(node["ref"], ref)
                self.assertEqual(node["package_id"], package_id)
                self.assertEqual(node.get("prev"), None)  # PREV is not defined yet, only exported

        client.run("lock build-order conan.lock --json=bo.json")
        jsonbo = json.loads(client.load("bo.json"))
        self.assertEqual(build_order, jsonbo)

        if export:
            client.run("install app/0.1@ --lockfile=conan.lock", assert_error=True)
            self.assertIn("Missing prebuilt package for 'app/0.1', 'dep/0.1', 'pkg/0.1'", client.out)

        # Build one by one
        client.run("install {0} --lockfile=conan.lock --lockfile-out=conan.lock"
                   " --build={0}".format(build_order[0][0][0]))
        self.assertIn("dep/0.1:5ab84d6acfe1f23c4fae0ab88f26e3a396351ac9 - Build", client.out)
        locked = json.loads(client.load("conan.lock"))["graph_lock"]["nodes"]
        node = locked["3"]
        self.assertEqual(node.get("prev"), prev_dep)
        node = locked["2"]
        self.assertEqual(node.get("prev"), None)  # PREV is not defined yet, only exported
        node = locked["1"]
        self.assertEqual(node.get("prev"), None)  # PREV is not defined yet, only exported

        client.run("lock build-order conan.lock --json=bo.json")
        jsonbo = json.loads(client.load("bo.json"))
        self.assertEqual(build_order[1:], jsonbo)

        client.run("install pkg/0.1@ --lockfile=conan.lock --build", assert_error=True)
        rev = "#f3367e0e7d170aa12abccb175fee5f97" if client.cache.config.revisions_enabled else ""
        self.assertIn("Cannot build 'dep/0.1{}' because it is "
                      "already locked in the input lockfile".format(rev), client.out)
        client.run("install {0} --lockfile=conan.lock --lockfile-out=conan.lock "
                   "--build={0}".format(build_order[1][0][0]))
        self.assertIn("dep/0.1:5ab84d6acfe1f23c4fae0ab88f26e3a396351ac9 - Cache", client.out)
        self.assertIn("pkg/0.1:0b3845ce7fd8c0b4e46566097797bd872cb5bcf6 - Build", client.out)
        locked = json.loads(client.load("conan.lock"))["graph_lock"]["nodes"]
        node = locked["3"]
        self.assertEqual(node.get("prev"), prev_dep)
        node = locked["2"]
        self.assertEqual(node.get("prev"), prev_pkg)
        node = locked["1"]
        self.assertEqual(node.get("prev"), None)  # PREV is not defined yet, only exported

        client.run("lock build-order conan.lock --json=bo.json")
        jsonbo = json.loads(client.load("bo.json"))
        self.assertEqual(build_order[2:], jsonbo)

        client.run("install app/0.1@ --lockfile=conan.lock --build", assert_error=True)
        rev = "#f3367e0e7d170aa12abccb175fee5f97" if client.cache.config.revisions_enabled else ""
        self.assertIn("Cannot build 'dep/0.1{}' because it is "
                      "already locked in the input lockfile".format(rev), client.out)
        client.run("install {0} --lockfile=conan.lock --lockfile-out=conan.lock "
                   "--build={0}".format(build_order[2][0][0]))
        self.assertIn("dep/0.1:5ab84d6acfe1f23c4fae0ab88f26e3a396351ac9 - Cache", client.out)
        self.assertIn("pkg/0.1:0b3845ce7fd8c0b4e46566097797bd872cb5bcf6 - Cache", client.out)
        self.assertIn("app/0.1:745ccd40fd696b66b0cb160fd5251a533563bbb4 - Build", client.out)

        locked = json.loads(client.load("conan.lock"))["graph_lock"]["nodes"]
        node = locked["3"]
        self.assertEqual(node.get("prev"), prev_dep)
        node = locked["2"]
        self.assertEqual(node.get("prev"), prev_pkg)
        node = locked["1"]
        self.assertEqual(node.get("prev"), prev_app)

        # New build order, nothing else to do
        client.run("lock build-order conan.lock --json=bo.json")
        jsonbo = json.loads(client.load("bo.json"))
        self.assertEqual([], jsonbo)

    @pytest.mark.skipif(not get_env("TESTING_REVISIONS_ENABLED", False), reason="Only revisions")
    def test_package_revision_mode_build_order(self):
        # https://github.com/conan-io/conan/issues/6232
        client = TestClient()
        client.run("config set general.default_package_id_mode=package_revision_mode")
        client.save({"conanfile.py": GenConanfile()})
        client.run("export . libb/0.1@")
        client.run("export . libc/0.1@")
        client.save({"conanfile.py": GenConanfile().with_require("libc/0.1")})
        client.run("export . liba/0.1@")
        client.save({"conanfile.py": GenConanfile().with_require("liba/0.1")
                                                   .with_require("libb/0.1")})
        client.run("export . app/0.1@")

        client.run("lock create --reference=app/0.1@ --build=missing --lockfile-out=conan.lock")
        self.assertIn("app/0.1:Package_ID_unknown - Unknown", client.out)
        self.assertIn("liba/0.1:Package_ID_unknown - Unknown", client.out)
        self.assertIn("libb/0.1:5ab84d6acfe1f23c4fae0ab88f26e3a396351ac9 - Build", client.out)
        self.assertIn("libc/0.1:5ab84d6acfe1f23c4fae0ab88f26e3a396351ac9 - Build", client.out)
        lock = json.loads(client.load("conan.lock"))
        app = lock["graph_lock"]["nodes"]["1"]
        self.assertEqual(app["package_id"], "Package_ID_unknown")
        liba = lock["graph_lock"]["nodes"]["2"]
        self.assertEqual(liba["package_id"], "Package_ID_unknown")
        libc = lock["graph_lock"]["nodes"]["3"]
        self.assertEqual(libc["package_id"], "5ab84d6acfe1f23c4fae0ab88f26e3a396351ac9")
        libd = lock["graph_lock"]["nodes"]["4"]
        self.assertEqual(libd["package_id"], "5ab84d6acfe1f23c4fae0ab88f26e3a396351ac9")

        client.run("lock build-order conan.lock --json=bo.json")
        bo = client.load("bo.json")
        build_order = json.loads(bo)
        expected = [[['libc/0.1@#f3367e0e7d170aa12abccb175fee5f97',
                      '5ab84d6acfe1f23c4fae0ab88f26e3a396351ac9', 'host', '3'],
                     ['libb/0.1@#f3367e0e7d170aa12abccb175fee5f97',
                      '5ab84d6acfe1f23c4fae0ab88f26e3a396351ac9', 'host', '4']],
                    [['liba/0.1@#7086607aa6efbad8e2527748e3ee8237',
                      'Package_ID_unknown', 'host', '2']],
                    [['app/0.1@#7742ee9e2f19af4f9ed7619f231ca871',
                      'Package_ID_unknown', 'host', '1']]]
        self.assertEqual(build_order, expected)


class BuildRequiresBuildOrderTest(unittest.TestCase):

    @parameterized.expand([(True,), (False,)])
    def test_transitive_build_not_locked(self, export):
        # https://github.com/conan-io/conan/issues/5727
        client = TestClient()
        client.save({"dep/conanfile.py": GenConanfile(),
                     "pkg/conanfile.py": GenConanfile().with_build_requires("dep/0.1"),
                     "app/conanfile.py": GenConanfile().with_require("pkg/0.1")})
        if export:
            client.run("export dep dep/0.1@")
            client.run("export pkg pkg/0.1@")
            client.run("export app app/0.1@")
            # Necessary for build-requires
            client.run("lock create --reference app/0.1@ --build=missing --lockfile-out=conan.lock")
        else:
            client.run("create dep dep/0.1@")
            client.run("create pkg pkg/0.1@")
            client.run("create app app/0.1@")
            client.run("lock create --reference=app/0.1@ --build --lockfile-out=conan.lock")

        locked = json.loads(client.load("conan.lock"))["graph_lock"]["nodes"]

        if client.cache.config.revisions_enabled:
            build_order = [[["dep/0.1@#f3367e0e7d170aa12abccb175fee5f97",
                             "5ab84d6acfe1f23c4fae0ab88f26e3a396351ac9", "host", "3"]],
                           [["pkg/0.1@#1364f701b47130c7e38f04c5e5fab985",
                             "5ab84d6acfe1f23c4fae0ab88f26e3a396351ac9", "host", "2"]],
                           [["app/0.1@#5e0af887c3e9391c872773734ccd2ca0",
                             "a925a8281740e4cb4bcad9cf41ecc4c215210604", "host", "1"]]]
            prev_dep = "83c38d3b4e5f1b8450434436eec31b00"
            prev_pkg = "5d3d587702b55a456c9b6b71e5f40cfa"
            prev_app = "eeb6de9b69fb0905e15788315f77a8e2"

        else:
            build_order = [[["dep/0.1@",
                             "5ab84d6acfe1f23c4fae0ab88f26e3a396351ac9", "host", "3"]],
                           [["pkg/0.1@",
                             "5ab84d6acfe1f23c4fae0ab88f26e3a396351ac9", "host", "2"]],
                           [["app/0.1@",
                             "a925a8281740e4cb4bcad9cf41ecc4c215210604", "host", "1"]]]
            prev_dep = "0"
            prev_pkg = "0"
            prev_app = "0"

        for level in build_order:
            for item in level:
                ref, package_id, _, lockid = item
                ref = ref.replace("@", "")
                node = locked[lockid]
                self.assertEqual(node["ref"], ref)
                self.assertEqual(node["package_id"], package_id)
                self.assertEqual(node.get("prev"), None)  # PREV is not defined yet, only exported

        client.run("lock build-order conan.lock --json=bo.json")
        jsonbo = json.loads(client.load("bo.json"))
        self.assertEqual(build_order, jsonbo)

        # prev is None, --build needs to be explicit or it will fail
        if export:
            client.run("install app/0.1@ --lockfile=conan.lock", assert_error=True)
            self.assertIn("Missing prebuilt package for 'app/0.1', 'pkg/0.1'", client.out)

        # Build one by one
        client.run("install {0} --lockfile=conan.lock --lockfile-out=conan.lock "
                   "--build={0}".format(build_order[0][0][0]))
        self.assertIn("dep/0.1:5ab84d6acfe1f23c4fae0ab88f26e3a396351ac9 - Build", client.out)
        locked = json.loads(client.load("conan.lock"))["graph_lock"]["nodes"]
        node = locked["3"]
        self.assertEqual(node.get("prev"), prev_dep)
        node = locked["2"]
        self.assertEqual(node.get("prev"), None)  # PREV is not defined yet, only exported
        node = locked["1"]
        self.assertEqual(node.get("prev"), None)  # PREV is not defined yet, only exported

        client.run("lock build-order conan.lock --json=bo.json")
        jsonbo = json.loads(client.load("bo.json"))
        self.assertEqual(build_order[1:], jsonbo)

        client.run("install pkg/0.1@ --lockfile=conan.lock --build", assert_error=True)
        rrev = "#f3367e0e7d170aa12abccb175fee5f97" if client.cache.config.revisions_enabled else ""
        self.assertIn("Cannot build 'dep/0.1{}' because it is "
                      "already locked in the input lockfile".format(rrev), client.out)
        client.run("install {0} --lockfile=conan.lock --lockfile-out=conan.lock "
                   "--build={0}".format(build_order[1][0][0]))
        self.assertIn("dep/0.1:5ab84d6acfe1f23c4fae0ab88f26e3a396351ac9 - Cache", client.out)
        self.assertIn("pkg/0.1:5ab84d6acfe1f23c4fae0ab88f26e3a396351ac9 - Build", client.out)
        locked = json.loads(client.load("conan.lock"))["graph_lock"]["nodes"]
        node = locked["3"]
        self.assertEqual(node.get("prev"), prev_dep)
        node = locked["2"]
        self.assertEqual(node.get("prev"), prev_pkg)
        node = locked["1"]
        self.assertEqual(node.get("prev"), None)  # PREV is not defined yet, only exported

        client.run("lock build-order conan.lock --json=bo.json")
        jsonbo = json.loads(client.load("bo.json"))
        self.assertEqual(build_order[2:], jsonbo)

        client.run("install app/0.1@ --lockfile=conan.lock --build", assert_error=True)
        rrev = "#1364f701b47130c7e38f04c5e5fab985" if client.cache.config.revisions_enabled else ""
        self.assertIn("Cannot build 'pkg/0.1{}' because it is "
                      "already locked in the input lockfile".format(rrev), client.out)
        client.run("install {0} --lockfile=conan.lock --lockfile-out=conan.lock "
                   "--build={0}".format(build_order[2][0][0]))
        self.assertNotIn("dep/0.1", client.out)
        self.assertIn("pkg/0.1:5ab84d6acfe1f23c4fae0ab88f26e3a396351ac9 - Cache", client.out)
        self.assertIn("app/0.1:a925a8281740e4cb4bcad9cf41ecc4c215210604 - Build", client.out)

        locked = json.loads(client.load("conan.lock"))["graph_lock"]["nodes"]
        node = locked["3"]
        self.assertEqual(node.get("prev"), prev_dep)
        node = locked["2"]
        self.assertEqual(node.get("prev"), prev_pkg)
        node = locked["1"]
        self.assertEqual(node.get("prev"), prev_app)

        # New build order, nothing else to do
        client.run("lock build-order conan.lock --json=bo.json")
        jsonbo = json.loads(client.load("bo.json"))
        self.assertEqual([], jsonbo)


class GraphLockWarningsTestCase(unittest.TestCase):

    def test_override(self):
        client = TestClient()
        client.save({"harfbuzz.py": GenConanfile("harfbuzz", "1.0"),
                     "ffmpeg.py": GenConanfile("ffmpeg", "1.0").with_requirement("harfbuzz/[>=1.0]"),
                     "meta.py": GenConanfile("meta", "1.0").with_requirement("ffmpeg/1.0")
                                                           .with_requirement("harfbuzz/1.0")
                     })
        client.run("export harfbuzz.py")
        client.run("export ffmpeg.py")
        client.run("export meta.py")

        # Building the graphlock we get the message
        client.run("lock create meta.py --lockfile-out=conan.lock")
        self.assertIn("WARN: ffmpeg/1.0: requirement harfbuzz/[>=1.0] overridden by meta/1.0"
                      " to harfbuzz/1.0", client.out)

        # Using the graphlock there is no warning message
        client.run("lock build-order conan.lock")
        self.assertNotIn("overridden", client.out)
        self.assertNotIn("WARN", client.out)


class GraphLockBuildRequireErrorTestCase(unittest.TestCase):

    def test_build_requires_should_be_locked(self):
        # https://github.com/conan-io/conan/issues/5807
        # this is the recommended approach, build_requires should be locked from the beginning
        client = TestClient()
        client.save({"zlib.py": GenConanfile(),
                     "harfbuzz.py": GenConanfile().with_require("fontconfig/1.0"),
                     "fontconfig.py": GenConanfile(),
                     "ffmpeg.py": GenConanfile().with_build_requires("fontconfig/1.0",
                                                                     "harfbuzz/1.0"),
                     "variant.py": GenConanfile().with_requires("ffmpeg/1.0",
                                                                "fontconfig/1.0",
                                                                "harfbuzz/1.0",
                                                                "zlib/1.0")
                     })
        client.run("export zlib.py zlib/1.0@")
        client.run("export fontconfig.py fontconfig/1.0@")
        client.run("export harfbuzz.py harfbuzz/1.0@")
        client.run("export ffmpeg.py ffmpeg/1.0@")

        # Building the graphlock we get the message
        client.run("lock create variant.py --build cascade --build outdated "
                   "--lockfile-out=conan.lock")

        if client.cache.config.revisions_enabled:
            fmpe = "ffmpeg/1.0#5522e93e2abfbd455e6211fe4d0531a2"
            font = "fontconfig/1.0#f3367e0e7d170aa12abccb175fee5f97"
            harf = "harfbuzz/1.0#3172f5e84120f235f75f8dd90fdef84f"
            zlib = "zlib/1.0#f3367e0e7d170aa12abccb175fee5f97"
            expected = [[['fontconfig/1.0@#f3367e0e7d170aa12abccb175fee5f97',
                          '5ab84d6acfe1f23c4fae0ab88f26e3a396351ac9', 'host', '2'],
                        ['zlib/1.0@#f3367e0e7d170aa12abccb175fee5f97',
                         '5ab84d6acfe1f23c4fae0ab88f26e3a396351ac9', 'host', '4']],
                        [['harfbuzz/1.0@#3172f5e84120f235f75f8dd90fdef84f',
                          'ea61889683885a5517800e8ebb09547d1d10447a', 'host', '3']],
                        [['ffmpeg/1.0@#5522e93e2abfbd455e6211fe4d0531a2',
                          '5ab84d6acfe1f23c4fae0ab88f26e3a396351ac9', 'host', '1']]]
        else:
            fmpe = "ffmpeg/1.0"
            font = "fontconfig/1.0"
            harf = "harfbuzz/1.0"
            zlib = "zlib/1.0"
            expected = [[['fontconfig/1.0@',
                          '5ab84d6acfe1f23c4fae0ab88f26e3a396351ac9', 'host', '2'],
                        ['zlib/1.0@',
                         '5ab84d6acfe1f23c4fae0ab88f26e3a396351ac9', 'host', '4']],
                        [['harfbuzz/1.0@',
                          'ea61889683885a5517800e8ebb09547d1d10447a', 'host', '3']],
                        [['ffmpeg/1.0@',
                          '5ab84d6acfe1f23c4fae0ab88f26e3a396351ac9', 'host', '1']]]

        lock1 = client.load("conan.lock")
        lock = json.loads(lock1)
        nodes = lock["graph_lock"]["nodes"]
        self.assertEqual(7, len(nodes))
        self.assertEqual(fmpe, nodes["1"]["ref"])
        self.assertEqual(["5", "6"], nodes["1"]["build_requires"])
        self.assertEqual(font, nodes["2"]["ref"])
        self.assertEqual(harf, nodes["3"]["ref"])
        self.assertEqual(zlib, nodes["4"]["ref"])
        self.assertEqual(font, nodes["5"]["ref"])
        self.assertEqual(harf, nodes["6"]["ref"])

        client.run("lock build-order conan.lock --json=bo.json")
        self.assertNotIn("cannot be found in lockfile", client.out)
        lock2 = client.load("conan.lock")
        self.assertEqual(lock2, lock1)
        build_order = json.loads(client.load("bo.json"))
        self.assertEqual(expected, build_order)

    def test_build_requires_not_needed(self):
        client = TestClient()
        client.save({'tool/conanfile.py': GenConanfile(),
                     'libA/conanfile.py': GenConanfile().with_build_requires("tool/1.0"),
                     'App/conanfile.py': GenConanfile().with_require("libA/1.0")})
        client.run("create tool tool/1.0@")
        client.run("create libA libA/1.0@")
        client.run("create App app/1.0@")

        # Create the full lock create
        client.run("lock create --reference=app/1.0@ --build --lockfile-out=conan.lock")
        lock = json.loads(client.load("conan.lock"))["graph_lock"]["nodes"]
        app = lock["1"]
        liba = lock["2"]
        tool = lock["3"]
        if client.cache.config.revisions_enabled:
            self.assertEqual(app["ref"], "app/1.0#ac2e355bf59f54e838c9d2f1d8d1126c")
            self.assertEqual(liba["ref"], "libA/1.0#3fb401b4f9169fab06be253aa3fbcc1b")
            self.assertEqual(tool["ref"], "tool/1.0#f3367e0e7d170aa12abccb175fee5f97")
        else:
            self.assertEqual(app["ref"], "app/1.0")
            self.assertEqual(liba["ref"], "libA/1.0")
            self.assertEqual(tool["ref"], "tool/1.0")

        self.assertEqual(app["package_id"], "8a4d75100b721bfde375a978c780bf3880a22bab")
        self.assertIsNone(app.get("prev"))
        self.assertEqual(liba["package_id"], "5ab84d6acfe1f23c4fae0ab88f26e3a396351ac9")
        self.assertIsNone(liba.get("prev"))
        self.assertEqual(tool["package_id"], "5ab84d6acfe1f23c4fae0ab88f26e3a396351ac9")
        self.assertIsNone(tool.get("prev"))

        client.run("lock build-order conan.lock --json=bo.json")
        bo0 = client.load("bo.json")
        if client.cache.config.revisions_enabled:
            tool = "tool/1.0@#f3367e0e7d170aa12abccb175fee5f97"
            liba = "libA/1.0@#3fb401b4f9169fab06be253aa3fbcc1b"
            app = "app/1.0@#ac2e355bf59f54e838c9d2f1d8d1126c"
        else:
            tool = "tool/1.0@"
            liba = "libA/1.0@"
            app = "app/1.0@"
        expected = [
            [[tool, "5ab84d6acfe1f23c4fae0ab88f26e3a396351ac9", "host", "3"]],
            [[liba, "5ab84d6acfe1f23c4fae0ab88f26e3a396351ac9", "host", "2"]],
            [[app, "8a4d75100b721bfde375a978c780bf3880a22bab", "host", "1"]]
            ]
        self.assertEqual(expected, json.loads(bo0))

        client.run("create libA libA/2.0@ --lockfile=conan.lock", assert_error=True)
        self.assertIn("ERROR: Couldn't find 'libA/2.0' in lockfile", client.out)

        # Instead we export it and create a new lock create
        client.run("export libA libA/2.0@")
        client.run("lock create --reference=app/1.0@ --build=missing --lockfile-out=new.lock")
        new = client.load("new.lock")
        self.assertNotIn("libA/2.0", new)
        client.run("lock build-order new.lock --json=bo.json")
        self.assertEqual(json.loads(client.load("bo.json")), [])



import os
import textwrap
import time
import unittest

from parameterized import parameterized

from conans.model.ref import ConanFileReference
from conans.paths import CONANFILE
from conans.test.utils.tools import TestClient, GenConanfile


class PyRequiresExtendTest(unittest.TestCase):

    @staticmethod
    def _define_base(client):
        conanfile = textwrap.dedent("""
            from conans import ConanFile
            class MyConanfileBase(ConanFile):
                def source(self):
                    self.output.info("My cool source!")
                def build(self):
                    self.output.info("My cool build!")
                def package(self):
                    self.output.info("My cool package!")
                def package_info(self):
                    self.output.info("My cool package_info!")
            """)
        client.save({"conanfile.py": conanfile})
        client.run("export . base/1.1@user/testing")

    def test_reuse(self):
        client = TestClient(default_server_user=True)
        self._define_base(client)
        reuse = textwrap.dedent("""
            from conans import ConanFile
            class PkgTest(ConanFile):
                python_requires = "base/1.1@user/testing"
                python_requires_extend = "base.MyConanfileBase"
            """)
        client.save({"conanfile.py": reuse}, clean_first=True)
        client.run("create . Pkg/0.1@user/testing")
        self.assertIn("Pkg/0.1@user/testing: My cool source!", client.out)
        self.assertIn("Pkg/0.1@user/testing: My cool build!", client.out)
        self.assertIn("Pkg/0.1@user/testing: My cool package!", client.out)
        self.assertIn("Pkg/0.1@user/testing: My cool package_info!", client.out)

        client.run("upload * --all --confirm")
        client.run("remove * -f")
        client.run("install Pkg/0.1@user/testing")
        self.assertIn("Pkg/0.1@user/testing: My cool package_info!", client.out)
        client.run("remove * -f")
        client.run("download Pkg/0.1@user/testing")
        self.assertIn("Pkg/0.1@user/testing: Package installed "
                      "69265e58ddc68274e0c5510905003ff78c9db5de", client.out)

    def test_reuse_dot(self):
        client = TestClient(default_server_user=True)
        conanfile = textwrap.dedent("""
            from conans import ConanFile
            class MyConanfileBase(ConanFile):
                def build(self):
                    self.output.info("My cool build!")
            """)
        client.save({"conanfile.py": conanfile})
        client.run("export . my.base/1.1@user/testing")
        reuse = textwrap.dedent("""
            from conans import ConanFile
            class PkgTest(ConanFile):
                python_requires = "my.base/1.1@user/testing"
                python_requires_extend = "my.base.MyConanfileBase"
            """)
        client.save({"conanfile.py": reuse}, clean_first=True)
        client.run("create . Pkg/0.1@user/testing")
        self.assertIn("Pkg/0.1@user/testing: My cool build!", client.out)

    def test_with_alias(self):
        client = TestClient()
        self._define_base(client)
        client.run("alias base/LATEST@user/testing base/1.1@user/testing")

        reuse = textwrap.dedent("""
            from conans import ConanFile
            class PkgTest(ConanFile):
                python_requires = "base/LATEST@user/testing"
                python_requires_extend = "base.MyConanfileBase"
            """)
        client.save({"conanfile.py": reuse}, clean_first=True)
        client.run("create . Pkg/0.1@user/testing")
        self.assertIn("Pkg/0.1@user/testing: My cool source!", client.out)
        self.assertIn("Pkg/0.1@user/testing: My cool build!", client.out)
        self.assertIn("Pkg/0.1@user/testing: My cool package!", client.out)
        self.assertIn("Pkg/0.1@user/testing: My cool package_info!", client.out)

    def test_reuse_version_ranges(self):
        client = TestClient()
        self._define_base(client)

        reuse = textwrap.dedent("""
            from conans import ConanFile
            class PkgTest(ConanFile):
                python_requires = "base/[>1.0,<1.2]@user/testing"
                python_requires_extend = "base.MyConanfileBase"
            """)

        client.save({"conanfile.py": reuse}, clean_first=True)
        client.run("create . Pkg/0.1@user/testing")
        self.assertIn("Python requires", str(client.out).splitlines())
        self.assertIn("    base/1.1@user/testing", str(client.out).splitlines())
        self.assertIn("Pkg/0.1@user/testing: My cool source!", client.out)
        self.assertIn("Pkg/0.1@user/testing: My cool build!", client.out)
        self.assertIn("Pkg/0.1@user/testing: My cool package!", client.out)
        self.assertIn("Pkg/0.1@user/testing: My cool package_info!", client.out)

    def test_multiple_reuse(self):
        client = TestClient()
        conanfile = textwrap.dedent("""
            from conans import ConanFile
            class SourceBuild(ConanFile):
                def source(self):
                    self.output.info("My cool source!")
                def build(self):
                    self.output.info("My cool build!")
            """)
        client.save({"conanfile.py": conanfile})
        client.run("export . SourceBuild/1.0@user/channel")

        conanfile = textwrap.dedent("""
            from conans import ConanFile
            class PackageInfo(ConanFile):
                def package(self):
                    self.output.info("My cool package!")
                def package_info(self):
                    self.output.info("My cool package_info!")
            """)
        client.save({"conanfile.py": conanfile})
        client.run("export . PackageInfo/1.0@user/channel")

        conanfile = textwrap.dedent("""
            from conans import ConanFile
            class MyConanfileBase(ConanFile):
                python_requires = "SourceBuild/1.0@user/channel", "PackageInfo/1.0@user/channel"
                python_requires_extend = "SourceBuild.SourceBuild", "PackageInfo.PackageInfo"
            """)
        client.save({"conanfile.py": conanfile})
        client.run("create . Pkg/0.1@user/testing")
        self.assertIn("Pkg/0.1@user/testing: My cool source!", client.out)
        self.assertIn("Pkg/0.1@user/testing: My cool build!", client.out)
        self.assertIn("Pkg/0.1@user/testing: My cool package!", client.out)
        self.assertIn("Pkg/0.1@user/testing: My cool package_info!", client.out)

    @staticmethod
    def test_transitive_access():
        client = TestClient()
        client.save({"conanfile.py": GenConanfile()})
        client.run("export . base/1.0@user/channel")

        conanfile = textwrap.dedent("""
            from conans import ConanFile
            class Helper(ConanFile):
                python_requires = "base/1.0@user/channel"
            """)
        client.save({"conanfile.py": conanfile})
        client.run("export . helper/1.0@user/channel")

        conanfile = textwrap.dedent("""
            from conans import ConanFile
            class Pkg(ConanFile):
                python_requires = "helper/1.0@user/channel"
                def build(self):
                    self.python_requires["base"]
            """)
        client.save({"conanfile.py": conanfile})
        client.run("create . pkg/0.1@user/channel")
        assert "pkg/0.1@user/channel: Created package" in client.out

        conanfile = textwrap.dedent("""
            from conans import ConanFile
            class Pkg(ConanFile):
                python_requires = "helper/1.0@user/channel"
                python_requires_extend = "base.HelloConan"
            """)
        client.save({"conanfile.py": conanfile})
        client.run("create . pkg/0.1@user/channel")
        assert "pkg/0.1@user/channel: Created package" in client.out

    def test_multiple_requires_error(self):
        client = TestClient()
        conanfile = textwrap.dedent("""
            from conans import ConanFile
            myvar = 123
            def myfunct():
                return 123
            class Pkg(ConanFile):
                pass
            """)
        client.save({"conanfile.py": conanfile})
        client.run("export . pkg1/1.0@user/channel")

        conanfile = textwrap.dedent("""
            from conans import ConanFile
            myvar = 234
            def myfunct():
                return 234
            class Pkg(ConanFile):
                pass
            """)
        client.save({"conanfile.py": conanfile})
        client.run("export . pkg2/1.0@user/channel")

        conanfile = textwrap.dedent("""
            from conans import ConanFile
            class MyConanfileBase(ConanFile):
                python_requires = "pkg1/1.0@user/channel", "pkg2/1.0@user/channel"
                def build(self):
                    self.output.info("PKG1 N: %s" % self.python_requires["pkg1"].conanfile.name)
                    self.output.info("PKG1 V: %s" % self.python_requires["pkg1"].conanfile.version)
                    self.output.info("PKG1 U: %s" % self.python_requires["pkg1"].conanfile.user)
                    self.output.info("PKG1 C: %s" % self.python_requires["pkg1"].conanfile.channel)
                    self.output.info("PKG1 : %s" % self.python_requires["pkg1"].module.myvar)
                    self.output.info("PKG2 : %s" % self.python_requires["pkg2"].module.myvar)
                    self.output.info("PKG1F : %s" % self.python_requires["pkg1"].module.myfunct())
                    self.output.info("PKG2F : %s" % self.python_requires["pkg2"].module.myfunct())
            """)
        client.save({"conanfile.py": conanfile})
        client.run("create . Consumer/0.1@user/testing")
        self.assertIn("Consumer/0.1@user/testing: PKG1 N: pkg1", client.out)
        self.assertIn("Consumer/0.1@user/testing: PKG1 V: 1.0", client.out)
        self.assertIn("Consumer/0.1@user/testing: PKG1 U: user", client.out)
        self.assertIn("Consumer/0.1@user/testing: PKG1 C: channel", client.out)
        self.assertIn("Consumer/0.1@user/testing: PKG1 : 123", client.out)
        self.assertIn("Consumer/0.1@user/testing: PKG2 : 234", client.out)
        self.assertIn("Consumer/0.1@user/testing: PKG1F : 123", client.out)
        self.assertIn("Consumer/0.1@user/testing: PKG2F : 234", client.out)

    def test_local_import(self):
        client = TestClient(default_server_user=True)
        conanfile = textwrap.dedent("""
            from conans import ConanFile
            import mydata
            class MyConanfileBase(ConanFile):
                exports = "*.py"
                def source(self):
                    self.output.info(mydata.src)
                def build(self):
                    self.output.info(mydata.build)
                def package(self):
                    self.output.info(mydata.pkg)
                def package_info(self):
                    self.output.info(mydata.info)
            """)
        mydata = textwrap.dedent("""
            src = "My cool source!"
            build = "My cool build!"
            pkg = "My cool package!"
            info = "My cool package_info!"
            """)
        client.save({"conanfile.py": conanfile,
                     "mydata.py": mydata})
        client.run("export . base/1.1@user/testing")
        reuse = textwrap.dedent("""
            from conans import ConanFile
            class PkgTest(ConanFile):
                python_requires = "base/1.1@user/testing"
                python_requires_extend = "base.MyConanfileBase"
            """)

        client.save({"conanfile.py": reuse}, clean_first=True)
        client.run("create . Pkg/0.1@user/testing")
        self.assertIn("Pkg/0.1@user/testing: My cool source!", client.out)
        self.assertIn("Pkg/0.1@user/testing: My cool build!", client.out)
        self.assertIn("Pkg/0.1@user/testing: My cool package!", client.out)
        self.assertIn("Pkg/0.1@user/testing: My cool package_info!", client.out)

        client.run("upload * --all --confirm")
        client.run("remove * -f")
        client.run("install Pkg/0.1@user/testing")
        self.assertIn("Pkg/0.1@user/testing: My cool package_info!", client.out)
        client.run("remove * -f")
        client.run("download Pkg/0.1@user/testing")
        self.assertIn("Pkg/0.1@user/testing: Package installed "
                      "69265e58ddc68274e0c5510905003ff78c9db5de", client.out)

    def test_reuse_class_members(self):
        client = TestClient()
        conanfile = textwrap.dedent("""
            from conans import ConanFile
            class MyConanfileBase(ConanFile):
                license = "MyLicense"
                author = "author@company.com"
                exports = "*.txt"
                exports_sources = "*.h"
                short_paths = True
                generators = "cmake"
            """)
        client.save({"conanfile.py": conanfile,
                     "header.h": "some content"})
        client.run("export . base/1.1@user/testing")

        reuse = textwrap.dedent("""
            from conans import ConanFile
            from conans.tools import load
            import os
            class PkgTest(ConanFile):
                python_requires = "base/1.1@user/testing"
                python_requires_extend = "base.MyConanfileBase"
                def build(self):
                    self.output.info("Exports sources! %s" % self.exports_sources)
                    self.output.info("HEADER CONTENT!: %s" % load("header.h"))
                    self.output.info("Short paths! %s" % self.short_paths)
                    self.output.info("License! %s" % self.license)
                    self.output.info("Author! %s" % self.author)
                    assert os.path.exists("conanbuildinfo.cmake")
            """)
        client.save({"conanfile.py": reuse,
                     "header.h": "pkg new header contents",
                     "other.txt": "text"})
        client.run("create . Pkg/0.1@user/testing")
        self.assertIn("Pkg/0.1@user/testing: Exports sources! *.h", client.out)
        self.assertIn("Pkg/0.1@user/testing exports: Copied 1 '.txt' file: other.txt",
                      client.out)
        self.assertIn("Pkg/0.1@user/testing exports_sources: Copied 1 '.h' file: header.h",
                      client.out)
        self.assertIn("Pkg/0.1@user/testing: Short paths! True", client.out)
        self.assertIn("Pkg/0.1@user/testing: License! MyLicense", client.out)
        self.assertIn("Pkg/0.1@user/testing: Author! author@company.com", client.out)
        self.assertIn("Pkg/0.1@user/testing: HEADER CONTENT!: pkg new header contents", client.out)
        ref = ConanFileReference.loads("Pkg/0.1@user/testing")
        self.assertTrue(os.path.exists(os.path.join(client.cache.package_layout(ref).export(),
                                                    "other.txt")))

    def test_reuse_system_requirements(self):
        # https://github.com/conan-io/conan/issues/7718
        client = TestClient()
        conanfile = textwrap.dedent("""
           from conans import ConanFile
           class MyConanfileBase(ConanFile):
               def system_requirements(self):
                   self.output.info("My system_requirements %s being called!" % self.name)
           """)
        client.save({"conanfile.py": conanfile})
        client.run("export . base/1.1@user/testing")
        reuse = textwrap.dedent("""
            from conans import ConanFile
            class PkgTest(ConanFile):
                python_requires = "base/1.1@user/testing"
                python_requires_extend = "base.MyConanfileBase"
            """)
        client.save({"conanfile.py": reuse}, clean_first=True)
        client.run("create . Pkg/0.1@user/testing")
        self.assertIn("Pkg/0.1@user/testing: My system_requirements Pkg being called!", client.out)

    def test_overwrite_class_members(self):
        client = TestClient()
        conanfile = textwrap.dedent("""
            from conans import ConanFile
            class MyConanfileBase(ConanFile):
                license = "MyLicense"
                author = "author@company.com"
                settings = "os", # tuple!
            """)
        client.save({"conanfile.py": conanfile})
        client.run("export . base/1.1@user/testing")

        reuse = textwrap.dedent("""
            from conans import ConanFile
            class PkgTest(ConanFile):
                license = "MIT"
                author = "frodo"
                settings = "arch", # tuple!
                python_requires = "base/1.1@user/testing"
                python_requires_extend = "base.MyConanfileBase"

                def init(self):
                    base = self.python_requires["base"].module.MyConanfileBase
                    self.settings = base.settings + self.settings
                    self.license = base.license

                def build(self):
                    self.output.info("License! %s" % self.license)
                    self.output.info("Author! %s" % self.author)
                    self.output.info("os: %s arch: %s" % (self.settings.get_safe("os"),
                                                          self.settings.arch))
            """)
        client.save({"conanfile.py": reuse})
        client.run("create . Pkg/0.1@user/testing -s os=Windows -s arch=armv7")
        self.assertIn("Pkg/0.1@user/testing: License! MyLicense", client.out)
        self.assertIn("Pkg/0.1@user/testing: Author! frodo", client.out)
        self.assertIn("Pkg/0.1@user/testing: os: Windows arch: armv7", client.out)

    def test_failure_init_method(self):
        client = TestClient()
        base = textwrap.dedent("""
            from conans import ConanFile
            class MyBase(object):
                settings = "os", "build_type", "arch"
                options = {"base_option": [True, False]}
                default_options = {"base_option": False}

            class BaseConanFile(ConanFile):
                pass
            """)
        client.save({"conanfile.py": base})
        client.run("export . base/1.0@")
        derived = textwrap.dedent("""
            from conans import ConanFile
            class DerivedConan(ConanFile):
                settings = "os", "build_type", "arch"

                python_requires = "base/1.0"
                python_requires_extend = 'base.MyBase'

                options = {"derived_option": [True, False]}
                default_options = {"derived_option": False}

                def init(self):
                    base = self.python_requires['base'].module.MyBase
                    self.options.update(base.options)
                    self.default_options.update(base.default_options)
                """)
        client.save({"conanfile.py": derived})
        client.run("create . pkg/0.1@ -o base_option=True")
        self.assertIn("pkg/0.1: Created package", client.out)

    def test_transitive_imports_conflicts(self):
        # https://github.com/conan-io/conan/issues/3874
        client = TestClient()
        conanfile = textwrap.dedent("""
            from conans import ConanFile
            import myhelper
            class SourceBuild(ConanFile):
                exports = "*.py"
            """)
        helper = textwrap.dedent("""
            def myhelp(output):
                output.info("MyHelperOutput!")
            """)
        client.save({"conanfile.py": conanfile,
                     "myhelper.py": helper})
        client.run("export . base1/1.0@user/channel")
        client.save({"myhelper.py": helper.replace("MyHelperOutput!", "MyOtherHelperOutput!")})
        client.run("export . base2/1.0@user/channel")

        conanfile = textwrap.dedent("""
            from conans import ConanFile

            class MyConanfileBase(ConanFile):
                python_requires = "base2/1.0@user/channel", "base1/1.0@user/channel"
                def build(self):
                    self.python_requires["base1"].module.myhelper.myhelp(self.output)
                    self.python_requires["base2"].module.myhelper.myhelp(self.output)
            """)
        # This should work, even if there is a local "myhelper.py" file, which could be
        # accidentaly imported (and it was, it was a bug)
        client.save({"conanfile.py": conanfile})
        client.run("create . Pkg/0.1@user/testing")
        self.assertIn("Pkg/0.1@user/testing: MyHelperOutput!", client.out)
        self.assertIn("Pkg/0.1@user/testing: MyOtherHelperOutput!", client.out)

        # Now, the same, but with "clean_first=True", should keep working
        client.save({"conanfile.py": conanfile}, clean_first=True)
        client.run("create . Pkg/0.1@user/testing")
        self.assertIn("Pkg/0.1@user/testing: MyHelperOutput!", client.out)
        self.assertIn("Pkg/0.1@user/testing: MyOtherHelperOutput!", client.out)

    def test_update(self):
        client = TestClient(default_server_user=True)
        conanfile = textwrap.dedent("""
            from conans import ConanFile
            somevar = 42
            class MyConanfileBase(ConanFile):
                pass
            """)
        client.save({"conanfile.py": conanfile})
        client.run("export . base/1.1@user/testing")
        client.run("upload * --confirm")

        client2 = TestClient(servers=client.servers, users={"default": [("user", "mypass")]})
        reuse = textwrap.dedent("""
            from conans import ConanFile
            class PkgTest(ConanFile):
                python_requires = "base/1.1@user/testing"
                python_requires_extend = "base.MyConanfileBase"
                def configure(self):
                    self.output.info("PYTHON REQUIRE VAR %s"
                                     % self.python_requires["base"].module.somevar)
        """)

        client2.save({"conanfile.py": reuse})
        client2.run("install .")
        self.assertIn("conanfile.py: PYTHON REQUIRE VAR 42", client2.out)

        client.save({"conanfile.py": conanfile.replace("42", "143")})
        time.sleep(1)  # guarantee time offset
        client.run("export . base/1.1@user/testing")
        client.run("upload * --confirm")

        client2.run("install . --update")
        self.assertIn("conanfile.py: PYTHON REQUIRE VAR 143", client2.out)

    def test_update_ranges(self):
        # Same as the above, but using a version range, and no --update
        # https://github.com/conan-io/conan/issues/4650#issuecomment-497464305
        client = TestClient(default_server_user=True)
        conanfile = textwrap.dedent("""
            from conans import ConanFile
            somevar = 42
            class MyConanfileBase(ConanFile):
                pass
            """)
        client.save({"conanfile.py": conanfile})
        client.run("export . base/1.1@user/testing")
        client.run("upload * --confirm")

        client2 = TestClient(servers=client.servers, users={"default": [("user", "password")]})
        reuse = textwrap.dedent("""
            from conans import ConanFile
            class PkgTest(ConanFile):
                python_requires = "base/[>1.0]@user/testing"
                python_requires_extend = "base.MyConanfileBase"
                def configure(self):
                    self.output.info("PYTHON REQUIRE VAR %s"
                                     % self.python_requires["base"].module.somevar)
        """)

        client2.save({"conanfile.py": reuse})
        client2.run("install .")
        self.assertIn("conanfile.py: PYTHON REQUIRE VAR 42", client2.out)

        client.save({"conanfile.py": conanfile.replace("42", "143")})
        # Make sure to bump the version!
        client.run("export . base/1.2@user/testing")
        client.run("upload * --confirm")

        client2.run("install . --update")
        self.assertIn("conanfile.py: PYTHON REQUIRE VAR 143", client2.out)

    def test_duplicate_pyreq(self):
        t = TestClient()
        conanfile = textwrap.dedent("""
            from conans import ConanFile
            class PyReq(ConanFile):
                pass
        """)
        t.save({"conanfile.py": conanfile})
        t.run("export . pyreq/1.0@user/channel")
        t.run("export . pyreq/2.0@user/channel")

        conanfile = textwrap.dedent("""
            from conans import ConanFile
            class Lib(ConanFile):
                python_requires = "pyreq/1.0@user/channel", "pyreq/2.0@user/channel"
        """)
        t.save({"conanfile.py": conanfile})
        t.run("create . name/version@user/channel", assert_error=True)
        self.assertIn("ERROR: Error loading conanfile", t.out)
        self.assertIn("The python_require 'pyreq' already exists", t.out)

    def test_local_build(self):
        client = TestClient()
        client.save({"conanfile.py": "var=42\n"+str(GenConanfile())})
        client.run("export . tool/0.1@user/channel")
        conanfile = textwrap.dedent("""
            from conans import ConanFile
            class MyConanfileBase(ConanFile):
                python_requires = "tool/0.1@user/channel"
                def source(self):
                    self.output.info("Pkg1 source: %s" % self.python_requires["tool"].module.var)
                def build(self):
                    self.output.info("Pkg1 build: %s" % self.python_requires["tool"].module.var)
                def package(self):
                    self.output.info("Pkg1 package: %s" % self.python_requires["tool"].module.var)
            """)
        client.save({"conanfile.py": conanfile})
        client.run("source .")
        self.assertIn("conanfile.py: Pkg1 source: 42", client.out)
        client.run("install .")
        client.run("build .")
        self.assertIn("conanfile.py: Pkg1 build: 42", client.out)
        client.run("package .")
        self.assertIn("conanfile.py: Pkg1 package: 42", client.out)
        client.run("export-pkg . pkg1/0.1@user/testing")

    def test_reuse_name_version(self):
        client = TestClient()
        conanfile = textwrap.dedent("""
            from conans import ConanFile
            from conans.tools import load
            import os

            class Source(object):
                def set_name(self):
                    self.name = load("name.txt")

                def set_version(self):
                    self.version = load("version.txt")

            class MyConanfileBase(ConanFile):
                pass
            """)
        client.save({"conanfile.py": conanfile})
        client.run("export . tool/0.1@user/channel")
        conanfile = textwrap.dedent("""
            from conans import ConanFile
            class MyConanfileBase(ConanFile):
                python_requires = "tool/0.1@user/channel"
                python_requires_extend = "tool.Source"
                def source(self):
                    self.output.info("Pkg1 source: %s:%s" % (self.name, self.version))
                def build(self):
                    self.output.info("Pkg1 build: %s:%s" % (self.name, self.version))
                def package(self):
                    self.output.info("Pkg1 package: %s:%s" % (self.name, self.version))
            """)
        client.save({"conanfile.py": conanfile,
                     "name.txt": "MyPkg",
                     "version.txt": "MyVersion"})
        client.run("export .")
        self.assertIn("MyPkg/MyVersion: A new conanfile.py version was exported", client.out)
        client.run("create .")
        self.assertIn("MyPkg/MyVersion: Pkg1 source: MyPkg:MyVersion", client.out)
        self.assertIn("MyPkg/MyVersion: Pkg1 build: MyPkg:MyVersion", client.out)
        self.assertIn("MyPkg/MyVersion: Pkg1 package: MyPkg:MyVersion", client.out)

    @parameterized.expand([(False, False), (True, False), (True, True), ])
    def test_python_requires_with_alias(self, use_alias, use_alias_of_alias):
        assert use_alias if use_alias_of_alias else True
        version_str = "latest2" if use_alias_of_alias else "latest" if use_alias else "1.0"
        client = TestClient()

        # Create python_requires
        client.save({CONANFILE: textwrap.dedent("""
            from conans import ConanFile
            class PythonRequires0(ConanFile):
                def build(self):
                    super(PythonRequires0, self).build()
                    self.output.info("PythonRequires0::build")
                    """)})
        client.run("export . python_requires0/1.0@user/test")
        client.run("alias python_requires0/latest@user/test python_requires0/1.0@user/test")
        client.run("alias python_requires0/latest2@user/test python_requires0/latest@user/test")

        # Create python requires, that require the previous one
        client.save({CONANFILE: textwrap.dedent("""
            from conans import ConanFile
            class PythonRequires1(ConanFile):
                python_requires = "python_requires0/{v}@user/test"
                python_requires_extend = "python_requires0.PythonRequires0"
                def build(self):
                    super(PythonRequires1, self).build()
                    self.output.info("PythonRequires1::build")
            """).format(v=version_str)})
        client.run("export . python_requires1/1.0@user/test")
        client.run("alias python_requires1/latest@user/test python_requires1/1.0@user/test")
        client.run("alias python_requires1/latest2@user/test python_requires1/latest@user/test")

        # Create python requires
        client.save({CONANFILE: textwrap.dedent("""
            from conans import ConanFile
            class PythonRequires11(ConanFile):
                def build(self):
                    super(PythonRequires11, self).build()
                    self.output.info("PythonRequires11::build")
                    """)})
        client.run("export . python_requires11/1.0@user/test")
        client.run("alias python_requires11/latest@user/test python_requires11/1.0@user/test")
        client.run("alias python_requires11/latest2@user/test python_requires11/latest@user/test")

        # Create python requires, that require the previous one
        client.save({CONANFILE: textwrap.dedent("""
            from conans import ConanFile
            class PythonRequires22(ConanFile):
                python_requires = "python_requires0/{v}@user/test"
                python_requires_extend = "python_requires0.PythonRequires0"
                def build(self):
                    super(PythonRequires22, self).build()
                    self.output.info("PythonRequires22::build")
                    """).format(v=version_str)})
        client.run("export . python_requires22/1.0@user/test")
        client.run("alias python_requires22/latest@user/test python_requires22/1.0@user/test")
        client.run("alias python_requires22/latest2@user/test python_requires22/latest@user/test")

        # Another python_requires, that requires the previous python requires
        client.save({CONANFILE: textwrap.dedent("""
            from conans import ConanFile
            class PythonRequires2(ConanFile):
                python_requires = "python_requires1/{v}@user/test", "python_requires11/{v}@user/test"
                python_requires_extend = ("python_requires1.PythonRequires1",
                                      "python_requires11.PythonRequires11")
                def build(self):
                    super(PythonRequires2, self).build()
                    self.output.info("PythonRequires2::build")
                    """).format(v=version_str)})
        client.run("export . python_requires2/1.0@user/test")
        client.run("alias python_requires2/latest@user/test python_requires2/1.0@user/test")
        client.run("alias python_requires2/latest2@user/test python_requires2/latest@user/test")

        # My project, will consume the latest python requires
        client.save({CONANFILE: textwrap.dedent("""
            from conans import ConanFile
            class Project(ConanFile):
                python_requires = "python_requires2/{v}@user/test", "python_requires22/{v}@user/test"
                python_requires_extend = ("python_requires2.PythonRequires2",
                                          "python_requires22.PythonRequires22")
                def build(self):
                    super(Project, self).build()
                    self.output.info("Project::build")
                    """).format(v=version_str)})

        client.run("create . project/1.0@user/test --build=missing")

        # Check that everything is being built
        self.assertIn("project/1.0@user/test: PythonRequires11::build", client.out)
        self.assertIn("project/1.0@user/test: PythonRequires0::build", client.out)
        self.assertIn("project/1.0@user/test: PythonRequires22::build", client.out)
        self.assertIn("project/1.0@user/test: PythonRequires1::build", client.out)
        self.assertIn("project/1.0@user/test: PythonRequires2::build", client.out)
        self.assertIn("project/1.0@user/test: Project::build", client.out)

        # Check that all the graph is printed properly
        #   - requirements
        self.assertIn("    project/1.0@user/test from local cache - Cache", client.out)
        #   - python requires
        self.assertIn("    python_requires11/1.0@user/test", client.out)
        self.assertIn("    python_requires0/1.0@user/test", client.out)
        self.assertIn("    python_requires22/1.0@user/test", client.out)
        self.assertIn("    python_requires1/1.0@user/test", client.out)
        self.assertIn("    python_requires2/1.0@user/test", client.out)
        #   - packages
        self.assertIn("    project/1.0@user/test:88cd9e14eae0af6c823ed619608b6883037e5cbc - Build",
                      client.out)

        #   - no mention to alias
        self.assertNotIn("alias", client.out)
        self.assertNotIn("alias2", client.out)

    def test_reuse_export_sources(self):
        client = TestClient()
        conanfile = textwrap.dedent("""
            from conans import ConanFile
            class MyConanfileBase(ConanFile):
                exports = "*"
            """)
        client.save({"conanfile.py": conanfile,
                     "file.h": "myheader",
                     "folder/other.h": "otherheader"})
        client.run("export . tool/0.1@user/channel")
        conanfile = textwrap.dedent("""
            from conans import ConanFile, load
            import os
            class MyConanfileBase(ConanFile):
                python_requires = "tool/0.1@user/channel"
                def source(self):
                    sources = self.python_requires["tool"].path
                    file_h = os.path.join(sources, "file.h")
                    other_h = os.path.join(sources, "folder/other.h")
                    self.output.info("Source: tool header: %s" % load(file_h))
                    self.output.info("Source: tool other: %s" % load(other_h))
                def build(self):
                    sources = self.python_requires["tool"].path
                    file_h = os.path.join(sources, "file.h")
                    other_h = os.path.join(sources, "folder/other.h")
                    self.output.info("Build: tool header: %s" % load(file_h))
                    self.output.info("Build: tool other: %s" % load(other_h))
                def package(self):
                    sources = self.python_requires["tool"].path
                    file_h = os.path.join(sources, "file.h")
                    other_h = os.path.join(sources, "folder/other.h")
                    self.output.info("Package: tool header: %s" % load(file_h))
                    self.output.info("Package: tool other: %s" % load(other_h))
            """)
        client.save({"conanfile.py": conanfile,
                     "name.txt": "MyPkg",
                     "version.txt": "MyVersion"})
        client.run("export . pkg/1.0@user/channel")
        self.assertIn("pkg/1.0@user/channel: A new conanfile.py version was exported", client.out)
        client.run("create . pkg/1.0@user/channel")
        self.assertIn("pkg/1.0@user/channel: Source: tool header: myheader", client.out)
        self.assertIn("pkg/1.0@user/channel: Source: tool other: otherheader", client.out)
        self.assertIn("pkg/1.0@user/channel: Build: tool header: myheader", client.out)
        self.assertIn("pkg/1.0@user/channel: Build: tool other: otherheader", client.out)
        self.assertIn("pkg/1.0@user/channel: Package: tool header: myheader", client.out)
        self.assertIn("pkg/1.0@user/channel: Package: tool other: otherheader", client.out)

        # The local flow
        client.run("install .")
        client.run("source .")
        self.assertIn("conanfile.py: Source: tool header: myheader", client.out)
        self.assertIn("conanfile.py: Source: tool other: otherheader", client.out)
        client.run("build .")
        self.assertIn("conanfile.py: Build: tool header: myheader", client.out)
        self.assertIn("conanfile.py: Build: tool other: otherheader", client.out)

    def test_reuse_exports(self):
        client = TestClient()
        conanfile = textwrap.dedent("""
            from conans import ConanFile
            class MyConanfileBase(ConanFile):
                exports = "*"
            """)
        client.save({"conanfile.py": conanfile,
                     "file.h": "myheader",
                     "folder/other.h": "otherheader"})
        client.run("editable add . tool/0.1@user/channel")

        conanfile = textwrap.dedent("""
            from conans import ConanFile, load
            import os
            class MyConanfileBase(ConanFile):
                python_requires = "tool/0.1@user/channel"
                def source(self):
                    sources = self.python_requires["tool"].path
                    file_h = os.path.join(sources, "file.h")
                    other_h = os.path.join(sources, "folder/other.h")
                    self.output.info("Source: tool header: %s" % load(file_h))
                    self.output.info("Source: tool other: %s" % load(other_h))
                def build(self):
                    sources = self.python_requires["tool"].path
                    file_h = os.path.join(sources, "file.h")
                    other_h = os.path.join(sources, "folder/other.h")
                    self.output.info("Build: tool header: %s" % load(file_h))
                    self.output.info("Build: tool other: %s" % load(other_h))
            """)

        client2 = TestClient(cache_folder=client.cache_folder)
        client2.save({"conanfile.py": conanfile,
                      "name.txt": "MyPkg",
                      "version.txt": "MyVersion"})

        # The local flow
        client2.run("install .")
        client2.run("source .")
        self.assertIn("conanfile.py: Source: tool header: myheader", client2.out)
        self.assertIn("conanfile.py: Source: tool other: otherheader", client2.out)
        client2.run("build .")
        self.assertIn("conanfile.py: Build: tool header: myheader", client2.out)
        self.assertIn("conanfile.py: Build: tool other: otherheader", client2.out)

    def test_build_id(self):
        client = TestClient(default_server_user=True)
        self._define_base(client)
        reuse = textwrap.dedent("""
            from conans import ConanFile
            class PkgTest(ConanFile):
                python_requires = "base/1.1@user/testing"
                python_requires_extend = "base.MyConanfileBase"
                def build_id(self):
                    pass
            """)
        client.save({"conanfile.py": reuse}, clean_first=True)
        client.run("create . Pkg/0.1@user/testing")
        self.assertIn("Pkg/0.1@user/testing: My cool source!", client.out)
        self.assertIn("Pkg/0.1@user/testing: My cool build!", client.out)
        self.assertIn("Pkg/0.1@user/testing: My cool package!", client.out)
        self.assertIn("Pkg/0.1@user/testing: My cool package_info!", client.out)


def test_transitive_python_requires():
    # https://github.com/conan-io/conan/issues/8546
    client = TestClient()
    conanfile = textwrap.dedent("""
        from conans import ConanFile
        myvar = 123
        def myfunct():
            return 234
        class SharedFunction(ConanFile):
            name = "shared-function"
            version = "1.0"
        """)
    client.save({"conanfile.py": conanfile})
    client.run("export . @user/channel")

    conanfile = textwrap.dedent("""
        from conans import ConanFile
        class BaseClass(ConanFile):
            name = "base-class"
            version = "1.0"
            python_requires = "shared-function/1.0@user/channel"
            def build(self):
                pyreqs = self.python_requires
                v = pyreqs["shared-function"].module.myvar  # v will be 123
                f = pyreqs["shared-function"].module.myfunct()  # f will be 234
                self.output.info("%s, %s" % (v, f))
        """)
    client.save({"conanfile.py": conanfile})
    client.run("export . user/channel")

    conanfile = textwrap.dedent("""
        from conans import ConanFile
        class Consumer(ConanFile):
            name = "consumer"
            version = "1.0"
            python_requires = "base-class/1.0@user/channel"
            python_requires_extend = "base-class.BaseClass"
        """)
    client.save({"conanfile.py": conanfile})
    client.run("create . @user/channel")
    assert "consumer/1.0@user/channel: Calling build()\nconsumer/1.0@user/channel: 123, 234" in \
           client.out


def test_transitive_diamond_python_requires():
    client = TestClient()
    conanfile = textwrap.dedent("""
        from conans import ConanFile
        myvar = 123
        def myfunct():
            return 234
        class SharedFunction(ConanFile):
            name = "shared-function"
            version = "1.0"
        """)
    client.save({"conanfile.py": conanfile})
    client.run("export . @user/channel")

    conanfile = textwrap.dedent("""
        from conans import ConanFile
        myvar = 222
        def myfunct():
            return 2222
        class SharedFunction2(ConanFile):
            name = "shared-function2"
            version = "1.0"
        """)
    client.save({"conanfile.py": conanfile})
    client.run("export . @user/channel")

    conanfile = textwrap.dedent("""
        from conans import ConanFile
        class BaseClass(ConanFile):
            name = "base-class"
            version = "1.0"
            python_requires = "shared-function/1.0@user/channel", "shared-function2/1.0@user/channel"
            def build(self):
                pyreqs = self.python_requires
                v = pyreqs["shared-function"].module.myvar  # v will be 123
                f = pyreqs["shared-function2"].module.myfunct()  # f will be 2222
                self.output.info("%s, %s" % (v, f))
        """)
    client.save({"conanfile.py": conanfile})
    client.run("export . user/channel")

    conanfile = textwrap.dedent("""
        from conans import ConanFile
        class BaseClass2(ConanFile):
            name = "base-class2"
            version = "1.0"
            python_requires = "shared-function/1.0@user/channel", "shared-function2/1.0@user/channel"
            def package(self):
                pyreqs = self.python_requires
                v = pyreqs["shared-function2"].module.myvar  # v will be 222
                f = pyreqs["shared-function"].module.myfunct()  # f will be 234
                self.output.info("%s, %s" % (v, f))
        """)
    client.save({"conanfile.py": conanfile})
    client.run("export . user/channel")

    conanfile = textwrap.dedent("""
        from conans import ConanFile
        class Consumer(ConanFile):
            name = "consumer"
            version = "1.0"
            python_requires = "base-class/1.0@user/channel", "base-class2/1.0@user/channel"
            python_requires_extend = "base-class.BaseClass", "base-class2.BaseClass2"
        """)
    client.save({"conanfile.py": conanfile})
    client.run("create . @user/channel")
    assert "consumer/1.0@user/channel: Calling build()\nconsumer/1.0@user/channel: 123, 2222" in \
           client.out
    assert "consumer/1.0@user/channel: Calling package()\nconsumer/1.0@user/channel: 222, 234" in \
           client.out


def test_multiple_reuse():
    """ test how to enable the multiple code reuse for custom user generators
        # https://github.com/conan-io/conan/issues/11589
    """

    c = TestClient()
    common = textwrap.dedent("""
        from conan import ConanFile
        def mycommon():
            return 42
        class Common(ConanFile):
            name = "common"
            version = "0.1"
        """)
    tool = textwrap.dedent("""
        from conan import ConanFile

        class MyGenerator:
            common = None
            def __init__(self, conanfile):
                self.conanfile = conanfile
            def generate(self):
                self.conanfile.output.info("VALUE TOOL: {}!!!".format(MyGenerator.common.mycommon()))

        class Tool(ConanFile):
            name = "tool"
            version = "0.1"
            python_requires = "common/0.1"
            def init(self):
                MyGenerator.common = self.python_requires["common"].module
        """)
    consumer = textwrap.dedent("""
        from conan import ConanFile
        class Consumer(ConanFile):
            python_requires = "tool/0.1", "common/0.1"
            def generate(self):
                mycommon = self.python_requires["common"].module.mycommon
                self.output.info("VALUE COMMON: {}!!!".format(mycommon()))
                mygenerator = self.python_requires["tool"].module.MyGenerator(self)
                mygenerator.generate()
        """)
    c.save({"common/conanfile.py": common,
            "tool/conanfile.py": tool,
            "consumer/conanfile.py": consumer})
    c.run("export common")
    c.run("export tool")
    c.run("install consumer")
    assert "VALUE COMMON: 42!!!" in c.out
    assert "VALUE TOOL: 42!!!" in c.out



import os
import textwrap
import unittest

import pytest

from conans.client.tools import PkgConfig, environment_append
from conans.model.ref import ConanFileReference
from conans.test.assets.genconanfile import GenConanfile
from conans.test.utils.tools import TestClient


@pytest.mark.tool_pkg_config
class PkgConfigGeneratorWithComponentsTest(unittest.TestCase):

    @staticmethod
    def _create_greetings(client, custom_names=False, components=True):
        conanfile_greetings = textwrap.dedent("""
            from conans import ConanFile, CMake

            class GreetingsConan(ConanFile):
                name = "greetings"
                version = "0.0.1"
                settings = "os", "compiler", "build_type", "arch"

                def package_info(self):
                %s
            """)
        if components:
            info = textwrap.dedent("""
                        self.cpp_info.components["hello"].libs = ["hello"]
                        self.cpp_info.components["bye"].libs = ["bye"]
                        """)
            if custom_names:
                info += textwrap.dedent("""
                        self.cpp_info.names["pkg_config"] = "Greetings"
                        self.cpp_info.components["hello"].names["pkg_config"] = "Hello"
                        self.cpp_info.components["bye"].names["pkg_config"] = "Bye"
                        """)
        else:
            info = textwrap.dedent("""
                        self.cpp_info.libs = ["hello", "bye"]
                        """)
        wrapper = textwrap.TextWrapper(width=81, initial_indent="   ", subsequent_indent="        ")
        conanfile_greetings = conanfile_greetings % wrapper.fill(info)
        client.save({"conanfile.py": conanfile_greetings})
        client.run("create .")

    @staticmethod
    def _create_world(client, conanfile=None):
        _conanfile_world = textwrap.dedent("""
            from conans import ConanFile, CMake

            class WorldConan(ConanFile):
                name = "world"
                version = "0.0.1"
                settings = "os", "compiler", "build_type", "arch"
                requires = "greetings/0.0.1"

                def package_info(self):
                    self.cpp_info.components["helloworld"].requires = ["greetings::hello"]
                    self.cpp_info.components["helloworld"].libs = ["helloworld"]
                    self.cpp_info.components["worldall"].requires = ["helloworld",
                                                                     "greetings::greetings"]
                    self.cpp_info.components["worldall"].libs = ["worldall"]
            """)
        client.save({"conanfile.py": conanfile or _conanfile_world})
        client.run("create .")

    @staticmethod
    def _get_libs_from_pkg_config(library, folder):
        with environment_append({"PKG_CONFIG_PATH": folder}):
            pconfig = PkgConfig(library)
            libs = pconfig.libs_only_l
        return libs

    def test_basic(self):
        client = TestClient()
        self._create_greetings(client)
        self._create_world(client)
        client.run("install world/0.0.1@ -g pkg_config")
        self.assertNotIn("Requires:", client.load("hello.pc"))
        self.assertNotIn("Requires:", client.load("bye.pc"))
        self.assertIn("Requires: bye hello", client.load("greetings.pc"))
        for f in ["hello.pc", "bye.pc", "greetings.pc", "world.pc", "helloworld.pc", "worldall.pc"]:
            self.assertIn("Version: 0.0.1", client.load(f))
        libs = self._get_libs_from_pkg_config("greetings", client.current_folder)
        self.assertListEqual(["-lbye", "-lhello"], libs)

    def test_pkg_config_general(self):
        client = TestClient()
        self._create_greetings(client, custom_names=True)

        conanfile = textwrap.dedent("""
            from conans import ConanFile, CMake

            class WorldConan(ConanFile):
                name = "world"
                version = "0.0.1"
                settings = "os", "compiler", "build_type", "arch"
                requires = "greetings/0.0.1"

                def package_info(self):
                    self.cpp_info.names["pkg_config"] = "World"
                    self.cpp_info.components["helloworld"].names["pkg_config"] = "Helloworld"
                    self.cpp_info.components["helloworld"].requires = ["greetings::hello"]
                    self.cpp_info.components["helloworld"].libs = ["Helloworld"]
                    self.cpp_info.components["worldall"].names["pkg_config"] = "Worldall"
                    self.cpp_info.components["worldall"].requires = ["greetings::bye", "helloworld"]
                    self.cpp_info.components["worldall"].libs = ["Worldall"]
        """)
        self._create_world(client, conanfile=conanfile)
        client.run("install world/0.0.1@ -g pkg_config")
        libs = self._get_libs_from_pkg_config("Worldall", client.current_folder)
        self.assertListEqual(["-lWorldall", "-lbye", "-lHelloworld", "-lhello"], libs)
        libs = self._get_libs_from_pkg_config("Helloworld", client.current_folder)
        self.assertListEqual(["-lHelloworld", "-lhello"], libs)
        libs = self._get_libs_from_pkg_config("World", client.current_folder)
        self.assertListEqual(["-lWorldall", "-lbye", "-lHelloworld", "-lhello"], libs)
        for f in ["Hello.pc", "Bye.pc", "Greetings.pc", "World.pc", "Helloworld.pc", "Worldall.pc"]:
            self.assertIn("Version: 0.0.1", client.load(f))

    def test_pkg_config_components(self):
        client = TestClient()
        self._create_greetings(client)
        conanfile2 = textwrap.dedent("""
            from conans import ConanFile, CMake

            class WorldConan(ConanFile):
                name = "world"
                version = "0.0.1"
                settings = "os", "compiler", "build_type", "arch"
                requires = "greetings/0.0.1"

                def package_info(self):
                    self.cpp_info.components["helloworld"].requires = ["greetings::hello"]
                    self.cpp_info.components["helloworld"].libs = ["helloworld"]
                    self.cpp_info.components["worldall"].requires = ["helloworld", "greetings::bye"]
                    self.cpp_info.components["worldall"].libs = ["worldall"]
        """)
        self._create_world(client, conanfile=conanfile2)
        client.run("install world/0.0.1@ -g pkg_config")
        libs = self._get_libs_from_pkg_config("helloworld", client.current_folder)
        self.assertListEqual(["-lhelloworld", "-lhello"], libs)
        libs = self._get_libs_from_pkg_config("worldall", client.current_folder)
        self.assertListEqual(["-lworldall", "-lhelloworld", "-lhello", "-lbye"], libs)
        world_pc = client.load("world.pc")
        self.assertIn("Requires: helloworld worldall", world_pc)
        libs = self._get_libs_from_pkg_config("world", client.current_folder)
        self.assertListEqual(["-lworldall", "-lhelloworld", "-lhello", "-lbye"], libs)
        for f in ["hello.pc", "bye.pc", "greetings.pc", "world.pc", "helloworld.pc", "worldall.pc"]:
            self.assertIn("Version: 0.0.1", client.load(f))

    def test_recipe_with_components_requiring_recipe_without_components(self):
        client = TestClient()
        self._create_greetings(client, components=False)

        conanfile = textwrap.dedent("""
            from conans import ConanFile, CMake

            class WorldConan(ConanFile):
                name = "world"
                version = "0.0.1"
                settings = "os", "compiler", "build_type", "arch"
                requires = "greetings/0.0.1"

                def package_info(self):
                    self.cpp_info.components["helloworld"].requires = ["greetings::greetings"]
                    self.cpp_info.components["helloworld"].libs = ["helloworld"]
                    self.cpp_info.components["worldall"].requires = ["helloworld",
                                                                     "greetings::greetings"]
                    self.cpp_info.components["worldall"].libs = ["worldall"]
            """)
        self._create_world(client, conanfile=conanfile)
        client.run("install world/0.0.1@ -g pkg_config")
        self.assertFalse(os.path.isfile(os.path.join(client.current_folder, "hello.pc")))
        self.assertFalse(os.path.isfile(os.path.join(client.current_folder, "bye.pc")))
        greetings_pc = client.load("greetings.pc")
        self.assertNotIn("Requires:", greetings_pc)
        libs = self._get_libs_from_pkg_config("greetings", client.current_folder)
        self.assertListEqual(["-lhello", "-lbye"], libs)
        libs = self._get_libs_from_pkg_config("world", client.current_folder)
        self.assertListEqual(["-lworldall", "-lhelloworld", "-lhello", "-lbye"], libs)
        libs = self._get_libs_from_pkg_config("helloworld", client.current_folder)
        self.assertListEqual(["-lhelloworld", "-lhello", "-lbye"], libs)
        libs = self._get_libs_from_pkg_config("worldall", client.current_folder)
        self.assertListEqual(["-lworldall", "-lhelloworld", "-lhello", "-lbye"], libs)
        for f in ["greetings.pc", "world.pc", "helloworld.pc", "worldall.pc"]:
            self.assertIn("Version: 0.0.1", client.load(f))

    def test_same_names(self):
        client = TestClient()
        conanfile_greetings = textwrap.dedent("""
            from conans import ConanFile, CMake

            class HelloConan(ConanFile):
                name = "hello"
                version = "0.0.1"
                settings = "os", "compiler", "build_type", "arch"

                def package_info(self):
                    self.cpp_info.components["global"].name = "hello"
                    self.cpp_info.components["global"].libs = ["hello"]
            """)
        client.save({"conanfile.py": conanfile_greetings})
        client.run("create .")
        client.run("install hello/0.0.1@ -g pkg_config")
        self.assertNotIn("Requires:", client.load("hello.pc"))
        self.assertIn("Version: 0.0.1", client.load("hello.pc"))

    def test_component_not_found_same_name_as_pkg_require(self):
        zlib = GenConanfile("zlib", "0.1").with_setting("build_type")\
            .with_generator("pkg_config")
        mypkg = GenConanfile("mypkg", "0.1").with_setting("build_type")\
            .with_generator("pkg_config")
        final = GenConanfile("final", "0.1").with_setting("build_type")\
            .with_generator("pkg_config")\
            .with_require(ConanFileReference("zlib", "0.1", None, None))\
            .with_require(ConanFileReference("mypkg", "0.1", None, None))\
            .with_package_info(cpp_info={"components": {"cmp": {"requires": ["mypkg::zlib",
                                                                             "zlib::zlib"]}}},
                               env_info={})
        consumer = GenConanfile("consumer", "0.1").with_setting("build_type")\
            .with_generator("pkg_config")\
            .with_requirement(ConanFileReference("final", "0.1", None, None))
        client = TestClient()
        client.save({"zlib.py": zlib, "mypkg.py": mypkg, "final.py": final, "consumer.py": consumer})
        client.run("create zlib.py")
        client.run("create mypkg.py")
        client.run("create final.py")
        client.run("install consumer.py", assert_error=True)
        self.assertIn("Component 'mypkg::zlib' not found in 'mypkg' package requirement", client.out)



import textwrap

from conans.test.utils.conan_v2_tests import ConanV2ModeTestCase


class ConanfileSourceTestCase(ConanV2ModeTestCase):
    """ Conan v2: Settings and options are not available in method 'source()' """

    def test_no_settings(self):
        # self.setting is not available in 'source'
        t = self.get_client()
        conanfile = textwrap.dedent("""
            from conans import ConanFile

            class Recipe(ConanFile):
                settings = "os",

                def source(self):
                    self.output.info("conanfile::source(): settings.os={}".format(self.settings.os))
        """)
        t.save({'conanfile.py': conanfile})
        t.run('create . name/version@ -s os=Linux', assert_error=True)
        self.assertIn("Conan v2 incompatible: 'self.settings' access in source() method is deprecated", t.out)

    def test_no_options(self):
        # self.setting is not available in 'source'
        t = self.get_client()
        conanfile = textwrap.dedent("""
            from conans import ConanFile

            class Recipe(ConanFile):
                options = {'shared': [True, False]}

                def source(self):
                    self.output.info("conanfile::source(): options.shared={}".format(self.options.shared))
        """)
        t.save({'conanfile.py': conanfile})
        t.run('create . name/version@ -o shared=False', assert_error=True)
        self.assertIn("Conan v2 incompatible: 'self.options' access in source() method is deprecated", t.out)



import os
import textwrap

from conans.test.utils.tools import TestClient


def test_msbuildtoolchain_props_with_extra_flags():
    """
    Simple test checking that conantoolchain_release_x64.props is adding all the expected
    flags and preprocessor definitions
    """
    profile = textwrap.dedent("""\
    include(default)
    [settings]
    arch=x86_64
    [conf]
    tools.build:cxxflags=["--flag1", "--flag2"]
    tools.build:cflags+=["--flag3", "--flag4"]
    tools.build:sharedlinkflags+=["--flag5"]
    tools.build:exelinkflags+=["--flag6"]
    tools.build:defines+=["DEF1", "DEF2"]
    """)
    client = TestClient(path_with_spaces=False)
    client.run("new hello/0.1 --template=msbuild_lib")
    client.save({
        "myprofile": profile
    })
    # Local flow works
    client.run("install . -pr myprofile -if=install")
    toolchain = client.load(os.path.join("conan", "conantoolchain_release_x64.props"))
    expected_cl_compile = """
    <ClCompile>
      <PreprocessorDefinitions>DEF1;DEF2;%(PreprocessorDefinitions)</PreprocessorDefinitions>
      <AdditionalOptions>--flag1 --flag2 --flag3 --flag4 %(AdditionalOptions)</AdditionalOptions>"""
    expected_link = """
    <Link>
      <AdditionalOptions>--flag5 --flag6 %(AdditionalOptions)</AdditionalOptions>
    </Link>"""
    expected_resource_compile = """
    <ResourceCompile>
      <PreprocessorDefinitions>DEF1;DEF2;%(PreprocessorDefinitions)</PreprocessorDefinitions>
      <AdditionalOptions>--flag1 --flag2 --flag3 --flag4 %(AdditionalOptions)</AdditionalOptions>
    </ResourceCompile>"""
    assert expected_cl_compile in toolchain
    assert expected_link in toolchain
    assert expected_resource_compile in toolchain



import platform
import textwrap

import pytest

from conans.test.assets.cmake import gen_cmakelists
from conans.test.assets.sources import gen_function_cpp
from conans.test.utils.tools import TestClient
from conans.util.files import save


@pytest.fixture
def client():
    c = TestClient()
    save(c.cache.new_config_path, "tools.env.virtualenv:auto_use=True")
    clang_profile = textwrap.dedent("""
        [settings]
        arch=armv7
        build_type=RelWithDebInfo
        compiler=clang
        compiler.libcxx=libstdc++11
        compiler.version=12
        os=VxWorks
        os.version=7

        [buildenv]
        CC=clang
        CXX=clang++
        """)
    conanfile = textwrap.dedent("""
        import os
        from conans import ConanFile
        from conan.tools.cmake import CMake, cmake_layout

        class Pkg(ConanFile):
            settings = "os", "compiler", "build_type", "arch"
            exports_sources = "*"
            generators = "CMakeToolchain"

            def layout(self):
                cmake_layout(self)

            def build(self):
                cmake = CMake(self)
                cmake.configure()
                cmake.build()
                cmd = os.path.join(self.cpp.build.bindirs[0], "my_app")
                self.run('readelf -s ' + cmd, env=["conanrunenv"])
        """)
    toolchain_file = textwrap.dedent("""
            set(vsb "/vsb") # location of VxWorks Source Build (vsb)
            set(CMAKE_LINKER "ldarm")
            set(CMAKE_CXX_LINK_EXECUTABLE "<CMAKE_LINKER> <CMAKE_CXX_LINK_FLAGS> <LINK_FLAGS> <OBJECTS> -o <TARGET> <LINK_LIBRARIES>" CACHE STRING "Workaround for clang, use GNU linker not clang/lld" FORCE)
            set(CMAKE_CXX_STANDARD_INCLUDE_DIRECTORIES
                ${vsb}/usr/h
                ${vsb}/usr/h/public
                ${vsb}/h/config
                ${vsb}/share/h/public
            )
            add_compile_definitions(
                ARMEL
                CPU=_VX_ARMARCH7
                _REENTRANT
                INET
                _VSB_CONFIG_FILE="${vsb}/h/config/vsbConfig.h"
            )
            add_compile_options(
                -fno-builtin
                -pipe
            )
            add_compile_definitions(
                _C99
                _HAS_C9X
                _VX_CPU=_VX_ARMARCH7
                __RTP__
                TOOL=llvm
                TOOL_FAMILY=llvm
                CPU_FAMILY=ARM
                __ELF__
                __vxworks
                __VXWORKS__
                _USE_INIT_ARRAY
            )
            add_compile_options(
                -fasm
                -fomit-frame-pointer
                -ffunction-sections
                -fdata-sections
                --target=arm-eabi
                -mabi=aapcs
                -mcpu=cortex-a9
                -mfloat-abi=hard
                -mfpu=vfpv3
                -mlittle-endian
                -mno-implicit-float
                -nostdinc++
                -nostdlibinc
                -march=armv7
                -gdwarf-3
                -mllvm
                -two-entry-phi-node-folding-threshold=2
            )
            add_link_options(
                --target2=rel
            )
            link_directories(
                ${vsb}/usr/lib/common
            )

            set(CMAKE_CXX_STANDARD 14)
            set(CMAKE_C_STANDARD_LIBRARIES "--start-group --as-needed -lllvm -lc -lc_internal --end-group")
            set(CMAKE_CXX_STANDARD_LIBRARIES "-lnet --start-group --as-needed -lc -lc_internal -lllvm -lcplusplus -lllvmcplus -ldl --end-group")
            set(CMAKE_EXE_LINKER_FLAGS "--defsym __wrs_rtp_base=0x80000000 -u __wr_need_frame_add -u __tls__ -T${vsb}/usr/ldscripts/rtp.ld -static -EL ${vsb}/usr/lib/common/crt0.o" CACHE STRING "" FORCE)
        """)
    c.save({"conanfile.py": conanfile,
            "clang": clang_profile,
            "toolchain-vxworks.cmake": toolchain_file,
            "CMakeLists.txt": gen_cmakelists(appname="my_app", appsources=["src/main.cpp"]),
            "src/main.cpp": gen_function_cpp(name="main")})
    return c


@pytest.mark.tool_cmake
@pytest.mark.tool_clang(version="12")
def test_clang_cmake_ninja(client):
    client.run("create . pkg/0.1@ -pr=clang -c tools.cmake.cmaketoolchain:generator=Ninja -c tools.cmake.cmaketoolchain:toolchain_file=../../toolchain-vxworks.cmake")
    assert 'cmake -G "Ninja"' in client.out
    assert "__wrs_rtp_" in client.out



import os
import platform
import unittest

from conans.test.utils.tools import TestClient, GenConanfile
from conans.util.files import load


class VirtualEnvPythonGeneratorTest(unittest.TestCase):

    def test_simple_value(self):
        client = TestClient()
        dep1 = """
import os
from conans import ConanFile

class BaseConan(ConanFile):
    name = "base"
    version = "0.1"

    def package_info(self):
        self.env_info.PYTHONPATH="/path/to/something"
        self.env_info.LD_LIBRARY_PATH="/path/ld_library"
        self.env_info.DYLD_LIBRARY_PATH="/path/dyld_library"
        self.env_info.PATH="/path/path"
        self.env_info.OTHER="23"
"""

        base = '''
[requires]
base/0.1
[generators]
virtualenv
    '''
        client.save({"conanfile.py": dep1})
        client.run("create . ")
        client.save({"conanfile.txt": base}, clean_first=True)
        client.run("install . -g virtualenv_python")

        if platform.system() != "Windows":
            contents = client.load("environment_run_python.sh.env")
            self.assertIn('PYTHONPATH="/path/to/something"${PYTHONPATH:+:$PYTHONPATH}', contents)
        else:
            contents = client.load("environment_run_python.bat.env")
            self.assertIn('PYTHONPATH=/path/to/something;%PYTHONPATH%', contents)
        self.assertNotIn("OTHER", contents)
        self.assertIn("PATH=", contents)
        self.assertIn("LD_LIBRARY_PATH=", contents)
        self.assertIn("DYLD_LIBRARY_PATH=", contents)

    def test_multiple_value(self):
            client = TestClient()
            dep1 = """
from conans import ConanFile

class BaseConan(ConanFile):
    name = "base"
    version = "0.1"

    def package_info(self):
        self.env_info.PYTHONPATH=["/path/to/something", "/otherpath"]
        self.env_info.OTHER="23"
"""

            base = '''
    [requires]
    base/0.1
    [generators]
    virtualenv
        '''
            client.save({"conanfile.py": dep1})
            client.run("create . ")
            client.save({"conanfile.txt": base}, clean_first=True)
            client.run("install . -g virtualenv_python")

            if platform.system() != "Windows":
                contents = client.load("environment_run_python.sh.env")
                self.assertIn('PYTHONPATH="/path/to/something":"/otherpath"'
                              '${PYTHONPATH:+:$PYTHONPATH}', contents)
            else:
                contents = client.load("environment_run_python.bat.env")
                self.assertIn('PYTHONPATH=/path/to/something;/otherpath;%PYTHONPATH%', contents)
            self.assertNotIn("OTHER", contents)

    def test_no_value_declared(self):
        client = TestClient()
        dep1 = GenConanfile()

        base = '''
[requires]
base/0.1
[generators]
virtualenv
    '''
        client.save({"conanfile.py": dep1})
        client.run("create . base/0.1@")
        client.save({"conanfile.txt": base}, clean_first=True)
        client.run("install . -g virtualenv_python")
        name = "activate_run_python.sh" if platform.system() != "Windows" else "activate_run_python.bat"
        contents = client.load(name)
        self.assertNotIn("PYTHONPATH", contents)
